static const char *RcsId = "";
//+=============================================================================
//
// file :         PiMercury.cpp
//
// description :  C++ source for the PiMercury and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                PiMercury are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: graziano $
//
// $Revision: 1.20 $
//
// $Log: PiMercury.cpp,v $
// Revision 1.20  2017-05-16 13:54:34  graziano
// tango 9
//
// Revision 1.18  2016-08-23 12:06:21  graziano
// changed ChannelXOut to Out0X and added In01 ... In04
//
// Revision 1.17  2015-08-24 09:38:45  graziano
// added Out01 - Out04 attributes
//
// Revision 1.16  2014-06-12 11:50:40  graziano
// fixed memory leak
//
// Revision 1.15  2013-10-16 13:45:29  graziano
// Device_4Impl and support for new C863_10
//
// Revision 1.14  2013-07-09 08:08:33  graziano
// added command LoadConfig
//
// Revision 1.12  2012-01-12 08:45:48  graziano
// added support for multiple devices
//
// Revision 1.11  2011/04/20 12:34:43  graziano
// supported new buffer modes
//
// Revision 1.10  2010/08/26 08:09:14  graziano
// added logs
//
// Revision 1.9  2009/08/21 13:40:51  graziano
// New libbufferrt
//
// Revision 1.8  2009/08/20 06:24:59  graziano
// New libbufferrt
//
// Revision 1.7  2009/05/27 13:35:44  graziano
// fixed bug in delete
//
// Revision 1.6  2009/05/27 09:35:04  graziano
// check in rt_mode if the buffer is updating, otherwise set an error message in the status string
//
// Revision 1.5  2009/04/24 13:42:54  graziano
// bug fixing
//
// Revision 1.4  2009/04/24 07:52:03  graziano
// bug fixing
//
// Revision 1.3  2009/04/23 09:07:02  graziano
// development of rt_mode using kernel module
//
// Revision 1.2  2009/04/14 12:40:26  graziano
// added rt_mode, scan mode, ...
//
// Revision 1.1  2008/11/19 10:28:46  graziano
// first version
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name       |  Method name
//	----------------------------------------
//  State              |  dev_state()
//  Status             |  dev_status()
//  Forward            |  forward()
//  Backward           |  backward()
//  Stop               |  stop()
//  Abort              |  abort()
//  Brake              |  brake()
//  GoHome             |  go_home()
//  GetVersion         |  get_version()
//  MotorON            |  motor_on()
//  MotorOFF           |  motor_off()
//  SetHome            |  set_home()
//  LimitSwitchEnable  |  limit_switch_enable()
//  FindEdge           |  find_edge()
//  LimitSwitchLevel   |  limit_switch_level()
//  GetPosition        |  get_position()
//  CheckTrigger       |  check_trigger()
//  StartScan          |  start_scan()
//
//===================================================================

#include <sys/time.h>
#include <tango.h>
#include <PiMercury.h>
#include <PiMercuryClass.h>
#include "mercurythread.h"
#ifndef _SERVER_RT
#include <fcntl.h>
#include <sys/ioctl.h>
#include "mercury_rt.h"
#include <sys/mman.h>
#endif
#include <check_get_rt_buffer.h>
namespace PiMercury_ns
{

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::PiMercury(string &s)
// 
// description : 	constructor for simulated PiMercury
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
PiMercury::PiMercury(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

PiMercury::PiMercury(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

PiMercury::PiMercury(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		PiMercury::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void PiMercury::delete_device()
{
	//	Delete device's allocated object
	abortflag = 1;
	int ret;
	sleep(1);
	ret = munmap(buf, vsize);
	if(ret == -1)
	{
		WARN_STREAM << "PiMercury::delete_device(): error unmapping rt_buffer" << endl;
	}
	else if(ret == 0)
	{
		DEBUG_STREAM << "PiMercury::delete_device(): successfully unmapped rt_buffer" << endl;
	}
	if(fd_buffer >= 0)
		close(fd_buffer);
	if(fd_mercury >= 0)
		close(fd_mercury);
	
	delete attr_Position_read;
	delete attr_Acceleration_read;
	delete attr_Speed_read;
	delete attr_TargetPosition_read;
	delete [] attr_Limits_read;
	delete attr_ScanSpeed_read;
	
	delete [] attr_AxisConfig_read;
	
	delete attr_In01_read;
	delete attr_In02_read;
	delete attr_In03_read;
	delete attr_In04_read;
	
	delete [] rt_pos_array.pos;
	delete [] rt_pos_array.ts;
	delete [] rt_pos_array.bunch;
	
	delete mutex;
	delete devserial;
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void PiMercury::init_device()
{
	INFO_STREAM << "PiMercury::PiMercury() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	abortflag = 0;
	homing = 0;
	init_homing = false;
	err_connection = false;
	scan_mode = false;
	checking_trigger = false;
	exiting_rt_mode = false;
	abort_rt_sleep = false;
	//rt_pos_array = NULL;
	//maxAllowedSpeed = 9091;
	useGCS = false;
	referenced = false;
	get_device_property();
	
	attr_Out01_read = &attr_Out01_write;
	attr_Out02_read = &attr_Out02_write;
	attr_Out03_read = &attr_Out03_write;
	attr_Out04_read = &attr_Out04_write;
	
	attr_In01_read = new Tango::DevBoolean();
	attr_In02_read = new Tango::DevBoolean();
	attr_In03_read = new Tango::DevBoolean();
	attr_In04_read = new Tango::DevBoolean();
	
	serial_line = -1;
	vector<string> parsed;
	string_explode(serialdevice, "/", &parsed);
	DEBUG_STREAM << __FUNCTION__ << " parsed.size()"<<parsed.size();
	if(parsed.size() == 3)
	{
		std::istringstream ss(parsed[2]);
		DEBUG_STREAM << __FUNCTION__ << " parsed[2]="<<string(parsed[2]);
		ss >> serial_line;
	}

	cmd_map.insert(make_pair(VERSION_GET, VERSION_11_GET));
	cmd_map.insert(make_pair(TELL_POSITION, TELL_11_POSITION));
	cmd_map.insert(make_pair(TELL_TARGET_POSITION, TELL_11_TARGET_POSITION));
	cmd_map.insert(make_pair(TELL_VELOCITY, TELL_11_VELOCITY));
	cmd_map.insert(make_pair(TELL_PROGRAMMED_VELOCITY, TELL_11_PROGRAMMED_VELOCITY));
	cmd_map.insert(make_pair(TELL_PROGRAMMED_ACCELERATION, TELL_11_PROGRAMMED_ACCELERATION));
	cmd_map.insert(make_pair(TELL_STATUS, TELL_11_STATUS));
	cmd_map.insert(make_pair(SET_VELOCITY, SET_11_VELOCITY));
	cmd_map.insert(make_pair(SET_ACCELERATION, SET_11_ACCELERATION));
	cmd_map.insert(make_pair(CMD_STOP, CMD_11_STOP));
	cmd_map.insert(make_pair(CMD_ABORT, CMD_11_ABORT));
	cmd_map.insert(make_pair(CMD_GO_HOME, CMD_11_GO_HOME));
	cmd_map.insert(make_pair(CMD_SET_HOME, CMD_11_SET_HOME));
	cmd_map.insert(make_pair(CMD_MOV_ABSOLUTE, CMD_11_MOV_ABSOLUTE));
	cmd_map.insert(make_pair(CMD_MOV_RELATIVE, CMD_11_MOV_RELATIVE));
	cmd_map.insert(make_pair(CMD_ON, CMD_11_SVO));
	cmd_map.insert(make_pair(CMD_OFF, CMD_11_SVO));
	cmd_map.insert(make_pair(CMD_BRAKE_ON, CMD_11_BRAKE));
	cmd_map.insert(make_pair(CMD_BRAKE_OFF, CMD_11_BRAKE));
	cmd_map.insert(make_pair(CMD_FIND_EDGE, CMD_11_FIND_EDGE));
	cmd_map.insert(make_pair(SET_CHANNEL_ON, SET_11_CHANNEL_ON));
	cmd_map.insert(make_pair(SET_CHANNEL_OFF, SET_11_CHANNEL_OFF));
	cmd_map.insert(make_pair(TELL_CHANNEL, TELL_11_CHANNEL));

#ifndef _SERVER_RT
	int ret;
	fd_mercury = open(rTMercurydevice.c_str(),O_RDWR);
	if(fd_mercury < 0)
	{
		ERROR_STREAM << "PiMercury::PiMercury(): Error opening " << rTMercurydevice << endl;
	}
	unsigned int val = (((unsigned int)serial_line << 16) & 0xffff0000) |  0;	//disable RT
	DEBUG_STREAM << __FUNCTION__ << " ioctl MERCURY_RT_ENABLE with val=0x"<<hex<<val<<" ((unsigned int)serial_line << 16)="<<((unsigned int)serial_line << 16)<<"serial_line="<<serial_line<<dec;
	if(fd_mercury >= 0)
	{
		ret =ioctl(fd_mercury,MERCURY_RT_ENABLE,&val);
		if(ret)
		{
			WARN_STREAM << "PiMercury::PiMercury(): error in ioctl! ret="<< ret << endl;
		}
	}
#endif

	//position_tmp = 0;
	//position_moving = 0;
	//position_valid = false;
	moving = false;
	rtmode = false;		//TODO: how init??
	if(rtArraySize > 0 && rtArraySize < 32568)		//max for the RP Mecury command
	{
		rt_pos_array.pos = new double[rtArraySize];
		rt_pos_array.ts = new Tango::TimeVal[rtArraySize];
		rt_pos_array.bunch = new unsigned int[rtArraySize];
	}
	else
	{
		ERROR_STREAM << "PiMercury::PiMercury() RtArraySize device property has to be between 0 and 32568, nowRtArraySize=" << rtArraySize << endl;
		exit(0);
	}
	rt_pos_array_index = 0;
	
	//set_change_event("State", true, true);		//attribute name, implemented, detect
	
	Tango::Database *db = new Tango::Database();
	Tango::DbData db_data;	
	try {
		db_data.push_back(Tango::DbDatum("timeout"));
		string in_prop("timeout");
//		devserial->get_property(in_prop, db_data);
		db->get_device_property(serialdevice,db_data);
		db_data[0] >> serial_timeout;
		delete db;
	}
	catch(Tango::DevFailed e) {
		string temp;
		temp = e.errors[0].desc;
		WARN_STREAM << "PiMercury::PiMercury() error getting timeout property from serial device: " << temp << endl;
		delete db;		
	}
	if(serial_timeout < 0 || serial_timeout > 5000)
		serial_timeout = 500;
	
	/* connect to serialdevice */
	devserial = new Tango::DeviceProxy(serialdevice);
	readloop = new mercurythread(this);
	
	/* check if serialdevice is responding */
	try {
		if (devserial->ping()) 
		{
			devserial->set_timeout_millis(serial_timeout);
			set_state(Tango::UNKNOWN);	
			set_status("Connected to serialdevice " + serialdevice);
			old_state = device_state;
			old_status = device_status;
			INFO_STREAM << "Connected to serialdevice " + serialdevice << endl;
		}	
	} 
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Failed to connect to serialdevice " + serialdevice);
		old_state = device_state;
		old_status = device_status;
		INFO_STREAM << "Failed to connect to serialdevice " + serialdevice << endl; 			
	}
	
	Tango::DeviceData Din;
	Tango::DevLong flush = 2;  //flush input and output	
	Din << flush;
	try	{
		devserial->command_inout("DevSerFlush",Din);
	}
	catch(Tango::DevFailed e) {
		string temp;
		temp = e.errors[0].desc;
		WARN_STREAM << "PiMercury::PiMercury() error flushing serial device: " << temp << endl;
		err_connection = true;
	}
	
	attr_Position_read = new Tango::DevDouble();
	attr_Acceleration_read = new Tango::DevDouble();
	attr_Speed_read = new Tango::DevDouble();
	attr_TargetPosition_read = new Tango::DevDouble();
	attr_Limits_read = new Tango::DevDouble[2];
	attr_ScanSpeed_read = new Tango::DevDouble();
	
	attr_Limits_read[0] = axisBackwardLimit;
	attr_Limits_read[1] = axisForwardLimit;
/*	Tango::MultiAttribute *attr_list_ = get_device_attr();
	Tango::WAttribute &attr_lim = attr_list_->get_w_attr_by_name ("Limit");
	vector<double> limit_write;
	limit_write.push_back(attr_Limits_read[0]);
	limit_write.push_back(attr_Limits_read[1]);
	attr_lim.set_write_value(limit_write,2);*/ 
	attr_AxisConfig_read = new Tango::DevString[1024];

	 for (int i = 0; i < 1024; i++)
        {
                //ref_ptr[i] = ref_array[i];
                attr_AxisConfig_read[i] = conf_array[i];
        }
	
	mutex = new omni_mutex();
	
	//############# send address slection code ###############
	//read_Accuracy(Tango::Attribute &attr);
	string resp;
	char temp_c[32];
	c_86x_type = 0;
	if(useGCS)
	{
		c_86x_type = C_863_11;
		if(WriteReadMercury(IDN_11_GET,"",resp, true) < 0)
		{
			INFO_STREAM << "PiMercury::PiMercury(): exception IDN get: '"<< resp << "'" << endl;
		}
		else
		{
			if(resp.find("C-863.11") != string::npos)
			{
				c_86x_type = C_863_11;
				INFO_STREAM << "PiMercury::PiMercury(): found type of controller C-863.11!!" << endl;
			}
			else if(resp.find("C-863.12") != string::npos)
			{
				c_86x_type = C_863_12;
				INFO_STREAM << "PiMercury::PiMercury(): found type of controller C-863.12!!" << endl;
			}
		}
		abort_sleep(((double)wr_sleep)/1000);
	}
	if(c_86x_type < C_863_11)
	{
		sprintf(temp_c,"%ld",axisNumber-1);
		//val = strtol(temp.str().c_str(),0,16);
		//temp >> val;
		//abort_sleep(((double)wr_sleep)/1000);
		if(WriteReadMercury(ADDRESS_SELECTION_CODE,temp_c,resp, false) < 0)
			WARN_STREAM << "PiMercury::PiMercury(): exception address selection code: '"<< resp << "'" << endl;
		abort_sleep(((double)wr_sleep)/1000);		

		if(WriteReadMercury(VERSION_GET,"",resp, true) < 0)
		{
			WARN_STREAM << "PiMercury::PiMercury(): exception getting version: '"<< resp << "'" << endl;
		}
		else
		{
			//DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << VERSION_GET << " read response: " << resp << endl;
			version = string(resp.c_str());
			if(version.find("C-863") != string::npos)
			{
				c_86x_type = C_863;
				INFO_STREAM << "PiMercury::PiMercury(): found type of controller C-863!!" << endl;
			}
			/*else if(version.find("C-862") != string::npos)
			{
				c_86x_type = C_862;
				INFO_STREAM << "PiMercury::PiMercury(): found type of controller C-862!!" << endl;
			}*/
			else
			{
				c_86x_type = C_862;
				INFO_STREAM << "PiMercury::PiMercury(): type of controller not found in version, setting C-862!!" << endl;
			}
			DEBUG_STREAM << "PiMercury::PiMercury(): version=" << version << endl;
			strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
		}
	}
	else
	{
		if(WriteReadMercury(VERSION_11_GET,"",resp, true) < 0)
		{
			WARN_STREAM << "PiMercury::PiMercury(): exception getting version: '"<< resp << "'" << endl;
		}
		else
		{
			//DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << VERSION_GET << " read response: " << resp << endl;
			version = string(resp.c_str());
			DEBUG_STREAM << "PiMercury::PiMercury(): version=" << version << endl;
			//strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
		}
		abort_sleep(((double)wr_sleep)/1000);	
		if(WriteReadMercury(TELL_11_REFERENCED,"",resp, true) < 0)
		{
			WARN_STREAM << "PiMercury::PiMercury(): exception getting referenced: '"<< resp << "'" << endl;
		}
		else
		{
			int ax;
			int ref=0;
			if(sscanf(resp.c_str(), "%d=%d",&ax, &ref) == 2)
			{
				if(ref != 1)
				{
					referenced = false;
				}
				else
				{
					referenced = true;
				}
				DEBUG_STREAM << "PiMercury::PiMercury(): read referenced: " << ref << endl;
			}
		}
	}
	abort_sleep(((double)wr_sleep)/1000);
	if(c_86x_type == C_862)
	{
		if(WriteReadMercury(CMD_BRAKE_ON,"",resp, false) < 0)
			WARN_STREAM << "PiMercury::PiMercury(): exception setting brake ON: '"<< resp << "'" << endl;
		abort_sleep(((double)wr_sleep)/1000);	
	}
	else if(c_86x_type == C_863)
	{
		if(axisConfFile.length() > 0)
		{
			load_conf_file(axisConfFile);
			abort_sleep(((double)wr_sleep)/1000);
		}
	}
	else if(c_86x_type == C_863_11 || c_86x_type == C_863_12)
	{
/*		if(axisConfFile.length() > 0)
			load_conf_file(axisConfFile);
		abort_sleep(((double)wr_sleep)/1000);*/
		if(WriteReadMercury(CMD_ON,"1",resp, false) < 0)
			WARN_STREAM << "PiMercury::PiMercury(): exception setting servo loop ON: '"<< resp << "'" << endl;
		abort_sleep(((double)wr_sleep)/1000);
	}
	
	if(defaultMaxSpeed > 0 && defaultMaxSpeed <= maxAllowedSpeed)
	{
		double temp_d;
		temp_d = defaultMaxSpeed*positionRatio;
		sprintf(temp_c,"%d",(int)temp_d);		
		if(WriteReadMercury(SET_VELOCITY,temp_c,resp, false) < 0)
		{
			WARN_STREAM << "PiMercury::PiMercury(): exception setting default speed: '"<< resp << "'" << endl;
		}
		else
		{
			attr_Speed_write = defaultMaxSpeed;
			Tango::MultiAttribute *attr_list = get_device_attr();
			Tango::WAttribute &attr_ = attr_list->get_w_attr_by_name ("Speed"); 
			attr_.set_write_value(attr_Speed_write);
			DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << SET_VELOCITY << " read response: " << resp << endl;
		}
	}
	abort_sleep(((double)wr_sleep)/1000);
	if(defaultAcceleration > 0)
	{
		double temp_d;
		temp_d = defaultAcceleration*positionRatio;
		sprintf(temp_c,"%d",(int)temp_d);		
		if(WriteReadMercury(SET_ACCELERATION,temp_c,resp, false) < 0)
		{
			WARN_STREAM << "PiMercury::PiMercury(): exception setting default acceleration: '"<< resp << "'" << endl;
		}
		else
		{
			attr_Acceleration_write = defaultAcceleration;
			Tango::MultiAttribute *attr_list = get_device_attr();
			Tango::WAttribute &attr_ = attr_list->get_w_attr_by_name ("Acceleration"); 
			attr_.set_write_value(attr_Acceleration_write);
			DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << SET_ACCELERATION << " read response: " << resp << endl;
		}
	}
	sleep(1);
	abort_sleep(((double)wr_sleep)/1000);
	if(WriteReadMercury(TELL_POSITION,"",resp, true) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception telling position: '"<< resp << "'" << endl;
	}
	else if(resp.length() > 2)
	{
			int pos;
			sscanf(resp.c_str(), "P:%d",&pos);
			*attr_Position_read = (double)(pos) / positionRatio;
			DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << TELL_POSITION << " read response: " << resp << endl;
	}
	abort_sleep(((double)wr_sleep)/1000);
	if(WriteReadMercury(TELL_TARGET_POSITION,"",resp, true) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception telling position: '"<< resp << "'" << endl;
	}
	else if(resp.length() > 2)
	{
			int pos;
			sscanf(resp.c_str(), "T:%d",&pos);
			*attr_TargetPosition_read = (double)(pos) / positionRatio;
			attr_Position_write = *attr_TargetPosition_read;
			Tango::MultiAttribute *attr_list = get_device_attr();
			Tango::WAttribute &attr_ = attr_list->get_w_attr_by_name ("Position"); 
			attr_.set_write_value(attr_Position_write);
			DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << TELL_TARGET_POSITION << " read response: " << resp << endl;
	}
	abort_sleep(((double)wr_sleep)/1000);
	
		
/*	if(WriteReadMercury(TELL_BOARD,"",resp, true) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception telling board: '"<< resp << "'" << endl;
	}
	else
		DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << TELL_BOARD << " read response: " << resp << endl;
	abort_sleep(((double)wr_sleep)/1000);			
	if(WriteReadMercury(TELL_STATUS,"",resp, true) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception telling status: '"<< resp << "'" << endl;
	}
	else
		DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << TELL_STATUS << " read response: " << resp << endl;
	abort_sleep(((double)wr_sleep)/1000);			
	if(WriteReadMercury(TELL_POSITION,"",resp, true) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception telling position: '"<< resp << "'" << endl;
	}
	else
		DEBUG_STREAM << "PiMercury::PiMercury(): send command: " << TELL_POSITION << " read response: " << resp << endl;*/	
	
	
	/*if(WriteReadMercury("TP,WA2000,RP","",resp, true) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception telling TS,TP: '"<< resp << "'" << endl;
	}*/
/*	if(WriteReadMercury("LN","",resp, false) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception setting LN: '"<< resp << "'" << endl;
	}
	else
		DEBUG_STREAM << "PiMercury::PiMercury(): send command: LN read response: " << resp << endl;
	if(WriteReadMercury("LH","",resp, false) < 0)
	{
		WARN_STREAM << "PiMercury::PiMercury(): exception setting LH: '"<< resp << "'" << endl;
	}
	else
		DEBUG_STREAM << "PiMercury::PiMercury(): send command: LH read response: " << resp << endl;*/

	//TEST: try now to set brake off
	abort_sleep(((double)wr_sleep)/1000);
	if(c_86x_type == C_862)
	{
		if(WriteReadMercury(CMD_BRAKE_OFF,"",resp, false) < 0)
			WARN_STREAM << "PiMercury::PiMercury(): exception setting brake OFF: '"<< resp << "'" << endl;
		abort_sleep(((double)wr_sleep)/1000);	
	} 

#ifndef _SERVER_RT
	fd_buffer = open(rTBufferdevice.c_str(),O_RDWR);
	if(fd_buffer < 0)
	{
		ERROR_STREAM << "PiMercury::PiMercury(): Error opening " << rTBufferdevice << endl;
	}
	buf=0;
	ret =ioctl(fd_buffer,BUFFER_RT_IOC_READ_VSIZE,&vsize);
	if(ret)
	{
		ERROR_STREAM << "PiMercury::PiMercury(): error in ioctl! fd_buffer=" << fd_buffer << " ret="<< ret << endl;
		close(fd_buffer);
		//return -1;
	}
	DEBUG_STREAM << "PiMercury::PiMercury(): read from buffer vsize="<< vsize << endl;
	if(vsize <= 0)
	{
		close(fd_buffer);
		//return -1;
	}
	buf = (struct _buffer *) mmap(0, vsize, PROT_READ | PROT_WRITE, MAP_SHARED, fd_buffer, 0);
	
	if (buf == NULL)
	{
		ERROR_STREAM << "PiMercury::PiMercury(): error in mmap!!!" << endl;
		close(fd_buffer);
		//return -1;
	}
	DEBUG_STREAM << "PiMercury::PiMercury(): MMapped buffer!! buf=" << hex << buf << dec << endl;
#endif
	
	readloop->start();  
}


//+----------------------------------------------------------------------------
//
// method : 		PiMercury::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void PiMercury::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("AxisNumber"));
	dev_prop.push_back(Tango::DbDatum("AxisForwardLimit"));
	dev_prop.push_back(Tango::DbDatum("AxisBackwardLimit"));
	dev_prop.push_back(Tango::DbDatum("Serialdevice"));
	dev_prop.push_back(Tango::DbDatum("Serial_timeout"));
	dev_prop.push_back(Tango::DbDatum("Wr_sleep"));
	dev_prop.push_back(Tango::DbDatum("Th_sleep"));
	dev_prop.push_back(Tango::DbDatum("PositionRatio"));
	dev_prop.push_back(Tango::DbDatum("DefaultMaxSpeed"));
	dev_prop.push_back(Tango::DbDatum("DefaultMinSpeed"));
	dev_prop.push_back(Tango::DbDatum("DefaultAcceleration"));
	dev_prop.push_back(Tango::DbDatum("MaxAllowedSpeed"));
	dev_prop.push_back(Tango::DbDatum("RtArraySize"));
	dev_prop.push_back(Tango::DbDatum("ChannelTrigger"));
	dev_prop.push_back(Tango::DbDatum("Rt_Sleep"));
	dev_prop.push_back(Tango::DbDatum("HomeEdgeOffset"));
	dev_prop.push_back(Tango::DbDatum("RTMercurydevice"));
	dev_prop.push_back(Tango::DbDatum("RTBufferdevice"));
	dev_prop.push_back(Tango::DbDatum("UseGCS"));
	dev_prop.push_back(Tango::DbDatum("AxisConfFile"));
	dev_prop.push_back(Tango::DbDatum("C_863_10_new"));
	dev_prop.push_back(Tango::DbDatum("ReferenceSwitch"));
	dev_prop.push_back(Tango::DbDatum("RemoveEcho"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	PiMercuryClass	*ds_class =
		(static_cast<PiMercuryClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize AxisNumber from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axisNumber;
	else {
		//	Try to initialize AxisNumber from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axisNumber;
	}
	//	And try to extract AxisNumber value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisNumber;

	//	Try to initialize AxisForwardLimit from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axisForwardLimit;
	else {
		//	Try to initialize AxisForwardLimit from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axisForwardLimit;
	}
	//	And try to extract AxisForwardLimit value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisForwardLimit;

	//	Try to initialize AxisBackwardLimit from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axisBackwardLimit;
	else {
		//	Try to initialize AxisBackwardLimit from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axisBackwardLimit;
	}
	//	And try to extract AxisBackwardLimit value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisBackwardLimit;

	//	Try to initialize Serialdevice from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  serialdevice;
	else {
		//	Try to initialize Serialdevice from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  serialdevice;
	}
	//	And try to extract Serialdevice value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serialdevice;

	//	Try to initialize Serial_timeout from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  serial_timeout;
	else {
		//	Try to initialize Serial_timeout from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  serial_timeout;
	}
	//	And try to extract Serial_timeout value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serial_timeout;

	//	Try to initialize Wr_sleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  wr_sleep;
	else {
		//	Try to initialize Wr_sleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  wr_sleep;
	}
	//	And try to extract Wr_sleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  wr_sleep;

	//	Try to initialize Th_sleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  th_sleep;
	else {
		//	Try to initialize Th_sleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  th_sleep;
	}
	//	And try to extract Th_sleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  th_sleep;

	//	Try to initialize PositionRatio from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  positionRatio;
	else {
		//	Try to initialize PositionRatio from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  positionRatio;
	}
	//	And try to extract PositionRatio value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  positionRatio;

	//	Try to initialize DefaultMaxSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultMaxSpeed;
	else {
		//	Try to initialize DefaultMaxSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultMaxSpeed;
	}
	//	And try to extract DefaultMaxSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultMaxSpeed;

	//	Try to initialize DefaultMinSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultMinSpeed;
	else {
		//	Try to initialize DefaultMinSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultMinSpeed;
	}
	//	And try to extract DefaultMinSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultMinSpeed;

	//	Try to initialize DefaultAcceleration from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultAcceleration;
	else {
		//	Try to initialize DefaultAcceleration from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultAcceleration;
	}
	//	And try to extract DefaultAcceleration value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultAcceleration;

	//	Try to initialize MaxAllowedSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  maxAllowedSpeed;
	else {
		//	Try to initialize MaxAllowedSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  maxAllowedSpeed;
	}
	//	And try to extract MaxAllowedSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  maxAllowedSpeed;

	//	Try to initialize RtArraySize from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rtArraySize;
	else {
		//	Try to initialize RtArraySize from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rtArraySize;
	}
	//	And try to extract RtArraySize value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rtArraySize;

	//	Try to initialize ChannelTrigger from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  channelTrigger;
	else {
		//	Try to initialize ChannelTrigger from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  channelTrigger;
	}
	//	And try to extract ChannelTrigger value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  channelTrigger;

	//	Try to initialize Rt_Sleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rt_Sleep;
	else {
		//	Try to initialize Rt_Sleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rt_Sleep;
	}
	//	And try to extract Rt_Sleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rt_Sleep;

	//	Try to initialize HomeEdgeOffset from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  homeEdgeOffset;
	else {
		//	Try to initialize HomeEdgeOffset from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  homeEdgeOffset;
	}
	//	And try to extract HomeEdgeOffset value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  homeEdgeOffset;

	//	Try to initialize RTMercurydevice from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rTMercurydevice;
	else {
		//	Try to initialize RTMercurydevice from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rTMercurydevice;
	}
	//	And try to extract RTMercurydevice value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rTMercurydevice;

	//	Try to initialize RTBufferdevice from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rTBufferdevice;
	else {
		//	Try to initialize RTBufferdevice from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rTBufferdevice;
	}
	//	And try to extract RTBufferdevice value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rTBufferdevice;

	//	Try to initialize UseGCS from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  useGCS;
	else {
		//	Try to initialize UseGCS from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  useGCS;
	}
	//	And try to extract UseGCS value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  useGCS;

	//	Try to initialize AxisConfFile from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axisConfFile;
	else {
		//	Try to initialize AxisConfFile from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axisConfFile;
	}
	//	And try to extract AxisConfFile value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisConfFile;

	//	Try to initialize C_863_10_new from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  c_863_10_new;
	else {
		//	Try to initialize C_863_10_new from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  c_863_10_new;
	}
	//	And try to extract C_863_10_new value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  c_863_10_new;

	//	Try to initialize ReferenceSwitch from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  referenceSwitch;
	else {
		//	Try to initialize ReferenceSwitch from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  referenceSwitch;
	}
	//	And try to extract ReferenceSwitch value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  referenceSwitch;

	//	Try to initialize RemoveEcho from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  removeEcho;
	else {
		//	Try to initialize RemoveEcho from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  removeEcho;
	}
	//	And try to extract RemoveEcho value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  removeEcho;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		PiMercury::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void PiMercury::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void PiMercury::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "PiMercury::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}
//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_ScanSpeed
// 
// description : 	Extract real attribute values for ScanSpeed acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_ScanSpeed(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_ScanSpeed(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_ScanSpeed_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_ScanSpeed
// 
// description : 	Write ScanSpeed attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_ScanSpeed(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_ScanSpeed(Tango::WAttribute &attr) entering... "<< endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	attr.get_write_value(attr_ScanSpeed_write);
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::write_ScanSpeed()", Tango::ERR);	
	}
	if(attr_ScanSpeed_write > maxAllowedSpeed)
	{
		Tango::Except::throw_exception(
			(const char *) "Argument error",
			(const char *) "Speed value not allowed",
			(const char *) "PiMercury::write_ScanSpeed()", Tango::ERR);	
	}
	if(scan_mode)
	{
		string resp;
		char temp_c[32];
		double temp_d;
		temp_d = attr_ScanSpeed_write*positionRatio;
		sprintf(temp_c,"%d",(int)temp_d);
		if(WriteReadMercury(SET_VELOCITY,temp_c,resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::write_Speed()", Tango::ERR);
		}
	}
	else
	{
		*attr_ScanSpeed_read = attr_ScanSpeed_write;
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Limits
// 
// description : 	Extract real attribute values for Limits acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Limits(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Limits(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Limits_read,2);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Limits
// 
// description : 	Write Limits attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Limits(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Limits(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Limits_write);
	if(attr.get_write_value_length() != 2)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Expeting 2 limits: forward and backward",
			(const char *) "PiMercury::write_Limits()", Tango::ERR);	
	}
	attr_Limits_read[0] = attr_Limits_write[0];
	attr_Limits_read[1] = attr_Limits_write[1];
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_RtMode
// 
// description : 	Extract real attribute values for RtMode acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_RtMode(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_RtMode(Tango::Attribute &attr) entering... "<< endl;
	attr_RtMode_read = &rtmode;
	attr.set_value(attr_RtMode_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_RtMode
// 
// description : 	Write RtMode attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_RtMode(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_RtMode(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_RtMode_write);
	//if true update as soon as possible rtmode
	if(attr_RtMode_write)
	{
#ifndef _SERVER_RT	
		if(fd_mercury < 0)
		{
			TangoSys_MemStream out_stream;
			out_stream << "Error RT Module '" << rTMercurydevice << "' not opened!" << ends;
			Tango::Except::throw_exception(
				(const char *) "Error",
				out_stream.str(),
				(const char *) "PiMercury::write_RtMode()", Tango::ERR);
		}
#endif
		rtmode = attr_RtMode_write;	
	}
	
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::write_RtMode()", Tango::ERR);	
	}
	
	char temp_c[64];
	string resp;
	if(attr_RtMode_write)
	{
		string stat_tmp = get_status();
		set_status(stat_tmp + " | RT Mode");
		readloop->mutex_sleep->lock();
		readloop->pausesec_dfl = 0;					
		readloop->pausenano_dfl = rt_Sleep*1000000;
		readloop->mutex_sleep->unlock();
		DEBUG_STREAM << "PiMercury::write_RtMode(): changing to RT mode -> pausasec=" << readloop->pausesec_dfl << " pausanano=" << readloop->pausenano_dfl << endl;
#ifdef _SERVER_RT	
		mutex->lock();
		//temp_rt_resp = string("");
		rt_pos_array_index = 0;
		abort_rt_sleep = true;
		memset(temp_rt_resp,0,sizeof(temp_rt_resp));
		sprintf(temp_c,"WN%d,TP,WF%d,RP%d\r",channelTrigger,channelTrigger,rtArraySize);	//rtArraySize <= 32568 for RP command
		//if(WriteReadMercury(temp_c,"",resp, true) < 0)
		//{
		//	WARN_STREAM << "PiMercury::PiMercury(): exception telling TS,TP: '"<< resp << "'" << endl;
		//}	
		try {	
			write_serial(temp_c,strlen(temp_c));		
		}
		catch(Tango::DevFailed &e) {
			TangoSys_MemStream out_stream;
			out_stream << "Error writing cmd = " << temp_c << ", err = " << e.errors[0].desc << ends;
			WARN_STREAM << "PiMercury::write_RtMode(): " << out_stream.str() << endl;
			mutex->unlock();
			Tango::Except::throw_exception(
				(const char *) "Error",
				(const char *) "Not allowed in RT mode",
				(const char *) "PiMercury::write_RtMode()", Tango::ERR);						
		}
		mutex->unlock();
#else

		abort_rt_sleep = true;
		//DEBUG_STREAM << "PiMercury::write_RtMode(): before lock" << endl;
		mutex->lock();
		//DEBUG_STREAM << "PiMercury::write_RtMode(): after lock" << endl;
		//temp_rt_resp = string("");
		
		memset(temp_rt_resp,0,sizeof(temp_rt_resp));
		//sprintf(temp_c,"TP,WA5,RP%d\r",rtArraySize);	//rtArraySize <= 32568 for RP command
		if(c_86x_type < C_863_11)
		{
			sprintf(temp_c,"WN%d,TP,WF%d,RP\r",(int)channelTrigger,(int)channelTrigger);//wait ch ON, tell pos, wait ch OFF repeat
			//if(WriteReadMercury(temp_c,"",resp, true) < 0)
			//{
			//	WARN_STREAM << "PiMercury::PiMercury(): exception telling TS,TP: '"<< resp << "'" << endl;
			//}	
			try {	
				write_serial(temp_c,strlen(temp_c));		
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing cmd = " << temp_c << ", err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::write_RtMode(): " << out_stream.str() << endl;
				mutex->unlock();
				Tango::Except::throw_exception(
					(const char *) "Error",
					(const char *) "Not allowed in RT mode",
					(const char *) "PiMercury::write_RtMode()", Tango::ERR);						
			}
		}
		else
		{
#if 0
			/*MAC BEG TRG_LOOP
			WAC DIO? %d = 1
			POS? 1
			WAC DIO? %d = 0
			MAC START TRG_LOOP
			MAC END
			MAC START TRG_LOOP*/
			try {	
				sprintf(temp_c,"MAC BEG TRG_LOOP\n");
				write_serial(temp_c,strlen(temp_c));
				sprintf(temp_c,"WAC DIO? %d = 1\n",(int)channelTrigger);
				write_serial(temp_c,strlen(temp_c));
				sprintf(temp_c,"POS? %d\n",(int)axisNumber);
				write_serial(temp_c,strlen(temp_c));
				sprintf(temp_c,"WAC DIO? %d = 0\n",(int)channelTrigger);
				write_serial(temp_c,strlen(temp_c));
				sprintf(temp_c,"MAC START TRG_LOOP\n");
				write_serial(temp_c,strlen(temp_c));
				sprintf(temp_c,"MAC END\n");
				write_serial(temp_c,strlen(temp_c));
				sprintf(temp_c,"MAC START TRG_LOOP\n");
				write_serial(temp_c,strlen(temp_c));
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing cmd = " << temp_c << ", err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::write_RtMode(): " << out_stream.str() << endl;
				mutex->unlock();
				Tango::Except::throw_exception(
					(const char *) "Error",
					(const char *) "Not allowed in RT mode",
					(const char *) "PiMercury::write_RtMode()", Tango::ERR);						
			}
#endif
		}
		mutex->unlock();
		int ret;
		unsigned int val = ((channelTrigger << 2) & 0x0000ffff) | 0x01 | ((serial_line << 16) & 0xffff0000);
		if(fd_mercury >= 0)
		{
			DEBUG_STREAM << "PiMercury::write_RtMode(): going to write val=0x"<<hex << val<<dec << " with ioctl channelTrigger=" << channelTrigger << " channelTrigger << 2 =" << (channelTrigger << 2) << endl;
			ret =ioctl(fd_mercury,MERCURY_RT_ENABLE,&val);
			if(ret)
			{
				WARN_STREAM << "PiMercury::write_RtMode(): error in ioctl! ret="<< ret << endl;
			}
		}
#endif
	}
	else
	{
		abort_rt_sleep = false;
		int sleep_factor;
		if(get_state() == Tango::MOVING)
			sleep_factor = 1;
		else
			sleep_factor = 5;
		readloop->mutex_sleep->lock();
		readloop->pausesec_dfl = (int)(((double)(sleep_factor*th_sleep))/1000);					//poll status fast when not moving
		readloop->pausenano_dfl = (int)(((sleep_factor*th_sleep)-readloop->pausesec_dfl*1000)*1000000);		
		readloop->mutex_sleep->unlock();
		DEBUG_STREAM << "PiMercury::write_RtMode(): exiting RT mode -> pausasec=" << readloop->pausesec_dfl << " pausanano=" << readloop->pausenano_dfl << endl;
		string stat_tmp = get_status();
		int pos = stat_tmp.find("|");
		set_status(stat_tmp.substr(0,pos-1));
		exiting_rt_mode = true;
		
		if(c_86x_type == C_863_11 || c_86x_type == C_863_12)
		{
			try {
				sprintf(temp_c,"STP\n");	//to stop execution of macro
				write_serial(temp_c,strlen(temp_c));
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing cmd = " << temp_c << ", err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::write_RtMode(): " << out_stream.str() << endl;
				mutex->unlock();
				Tango::Except::throw_exception(
					(const char *) "Error",
					(const char *) "Not allowed in RT mode",
					(const char *) "PiMercury::write_RtMode()", Tango::ERR);						
			}
		}
		
#ifndef _SERVER_RT
		int ret;
		unsigned int val = 0 | ((serial_line << 16) & 0xffff0000);
		if(fd_mercury >= 0)
		{
			mutex->lock();
			memset(temp_rt_resp,0,sizeof(temp_rt_resp));
			sprintf(temp_c,"\r");
			try {	
				write_serial(temp_c,strlen(temp_c));		
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing cmd = " << temp_c << ", err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::write_RtMode(): " << out_stream.str() << endl;
				mutex->unlock();
				Tango::Except::throw_exception(
					(const char *) "Error",
					(const char *) "Not allowed in RT mode",
					(const char *) "PiMercury::write_RtMode()", Tango::ERR);						
			}
			mutex->unlock();
			
			ret =ioctl(fd_mercury,MERCURY_RT_ENABLE,&val);
			if(ret)
			{
				WARN_STREAM << "PiMercury::write_RtMode(): error in ioctl! ret="<< ret << endl;
			}
		}
		abort_sleep(0.2);
		Tango::DeviceData Din;
		Tango::DevLong flush = 0;  //flush RX	
		Din << flush;
		try	{
			devserial->command_inout("DevSerFlush",Din);
		}
		catch(Tango::DevFailed e) {
			TangoSys_MemStream out_stream;
			out_stream << "Error flushing serial device = " << e.errors[0].desc << ends;
			WARN_STREAM << "PiMercury::write_RtMode(): " << out_stream.str() << endl;		
		}
		abort_sleep(0.1);
#endif
		rtmode = false;	
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Error
// 
// description : 	Extract real attribute values for Error acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Error(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Error(Tango::Attribute &attr) entering... "<< endl;
	tmp_ptr = error_attr;
	attr_Error_read = &tmp_ptr;
	attr.set_value(attr_Error_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_TargetPosition
// 
// description : 	Extract real attribute values for TargetPosition acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_TargetPosition(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_TargetPosition(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_TargetPosition_read);	//TODO: add check_timeout ?
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Speed
// 
// description : 	Extract real attribute values for Speed acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Speed(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Speed(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Speed_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Speed
// 
// description : 	Write Speed attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Speed(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Speed(Tango::WAttribute &attr) entering... "<< endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	attr.get_write_value(attr_Speed_write);
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::write_Speed()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::write_speed()", Tango::ERR);	
	}
	if(attr_Speed_write > maxAllowedSpeed)
	{
		Tango::Except::throw_exception(
			(const char *) "Argument error",
			(const char *) "Speed value not allowed",
			(const char *) "PiMercury::write_Speed()", Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	double temp_d;
	temp_d = attr_Speed_write*positionRatio;
	if(c_86x_type < C_863_11)
		sprintf(temp_c,"%d",(int)temp_d);
	else
		sprintf(temp_c,"%f",temp_d);
	if(WriteReadMercury(SET_VELOCITY,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::write_Speed()", Tango::ERR);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Stat
// 
// description : 	Extract real attribute values for Stat acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Stat(Tango::Attribute &attr) entering... "<< endl;
	attr_Stat_read = &stat[0];
	attr.set_value(attr_Stat_read,STAT_SIZE);
}


//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Position
// 
// description : 	Extract real attribute values for Position acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Position(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Position(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Position_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Position
// 
// description : 	Write Position attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Position(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Position(Tango::WAttribute &attr) entering... "<< endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::write_Position()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::write_position()", Tango::ERR);	
	}
	if(get_state()==Tango::STANDBY)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put brake OFF before moving!",
			(const char *) "PiMercury::write_Position()", Tango::ERR);
	}
	else if(get_state()==Tango::OFF)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put motor loop ON before moving!",
			(const char *) "PiMercury::write_Position()", Tango::ERR);
	}
	else if(get_state()==Tango::DISABLE)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Reference Axis before moving!",
			(const char *) "PiMercury::write_Position()", Tango::ERR);
	}
	attr.get_write_value(attr_Position_write);
	if((attr_Position_write < attr_Limits_read[0]) || (attr_Position_write > attr_Limits_read[1]))
	{
		TangoSys_MemStream out_stream;
		out_stream << "Positions allowed are in the [" << attr_Limits_read[0] << "," << attr_Limits_read[1] << "] interval" << ends;
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			out_stream.str(),
			(const char *) "PiMercury::write_Position()", Tango::ERR);
	}
	string resp;
	double temp_d;
	char temp_c[32];
	temp_d = attr_Position_write*positionRatio;
	if(c_86x_type < C_863_11)
		sprintf(temp_c,"%d",(int)temp_d);
	else
		sprintf(temp_c,"%f",temp_d);
	INFO_STREAM << "PiMercury::write_Position writing user pos=" << attr_Position_write << " -> motor pos=" << temp_c << endl;
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	set_state(Tango::MOVING);
	readloop->mutex_sleep->unlock();	
	if(WriteReadMercury(CMD_MOV_ABSOLUTE,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::stop()", Tango::ERR);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Acceleration
// 
// description : 	Extract real attribute values for Acceleration acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Acceleration(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Acceleration(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Acceleration_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_AxisConfig
// 
// description : 	Extract real attribute values for AxisConfig acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_AxisConfig(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_AxisConfig(Tango::Attribute &attr) entering... "<< endl;
#if 0
	string resp;
	if(WriteReadMercury(TELL_11_AXIS_CONFIG,"",resp, true) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::read_AxisConfig()", Tango::ERR);
	}
	int i=0;
	vector<string> confs;
	string_explode(resp, "\n", &confs);
	for(vector<string>::iterator it=confs.begin(); it != confs.end(); it++)
	{
		strcpy(conf_array[i++], it->c_str());
	}
#endif
	char outbuf[256],
	inbuf[256];
	mutex->lock();
	sprintf(outbuf,"%s\n", TELL_11_AXIS_CONFIG);
	try {	
		write_serial(outbuf,strlen(outbuf));	
		abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)			
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Error writing SPA, err = " << e.errors[0].desc << ends;
		WARN_STREAM << "PiMercury::read_AxisConfig(): " << out_stream.str() << endl;
		mutex->unlock();
		return;							
	}
	bool err = false;
	int i=0;
	while(!err)
	{
		memset(inbuf,0,256);	
		try {	
			read_serial(inbuf);			
		}
		catch(Tango::DevFailed &e) {
			TangoSys_MemStream out_stream;
			out_stream << "Error cmd '" << outbuf << "' failed!!, err reading response= " << e.errors[0].desc << ends;
			WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
			err = true;
		}
		int resp_len = strlen(inbuf);
		if(resp_len > 0)
		{
			resp_len-=1;
			inbuf[resp_len]=0;
			strcpy(conf_array[i++], inbuf);
		}
		//abort_sleep(((double)wr_sleep)/1000);
	}
	mutex->unlock();
	attr.set_value(attr_AxisConfig_read, i);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Acceleration
// 
// description : 	Write Acceleration attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Acceleration(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Acceleration(Tango::WAttribute &attr) entering... "<< endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	attr.get_write_value(attr_Acceleration_write);
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::write_Acceleration()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::write_Acceleration()", Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	double temp_d;
	temp_d = attr_Acceleration_write*positionRatio;
	if(c_86x_type < C_863_11)
		sprintf(temp_c,"%d",(int)temp_d);
	else
		sprintf(temp_c,"%f",temp_d);
	if(WriteReadMercury(SET_ACCELERATION,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::write_Acceleration()", Tango::ERR);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Out01
// 
// description : 	Extract real attribute values for Out01 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Out01(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Out01(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Out01_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Out01
// 
// description : 	Write Out01 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Out01(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Out01(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Out01_write);
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;
	if(attr_Out01_write)
	{
		cmd = SET_CHANNEL_ON;
		if(!useGCS)
			sprintf(temp_c,"%d",1);
		else
			sprintf(temp_c,"%s","A 1");
	}
	else
	{
		cmd = SET_CHANNEL_OFF;
		if(!useGCS)
			sprintf(temp_c,"%d",1);
		else
			sprintf(temp_c,"%s","A 0");
	}
	cout << __func__<<": cmd="<<cmd<<" temp_c="<<temp_c<<endl;
	if(WriteReadMercury(cmd,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::write_Speed()", Tango::ERR);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Out02
// 
// description : 	Extract real attribute values for Out02 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Out02(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Out02(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Out02_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Out02
// 
// description : 	Write Out02 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Out02(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Out02(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Out02_write);
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;
	if(attr_Out02_write)
	{
		cmd = SET_CHANNEL_ON;
		if(!useGCS)
			sprintf(temp_c,"%d",2);
		else
			sprintf(temp_c,"%s","B 1");
	}
	else
	{
		cmd = SET_CHANNEL_OFF;
		if(!useGCS)
			sprintf(temp_c,"%d",2);
		else
			sprintf(temp_c,"%s","B 0");
	}
	if(WriteReadMercury(cmd,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::write_Speed()", Tango::ERR);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Out03
// 
// description : 	Extract real attribute values for Out03 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Out03(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Out03(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Out03_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Out03
// 
// description : 	Write Out03 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Out03(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Out03(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Out03_write);
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;
	if(attr_Out03_write)
	{
		cmd = SET_CHANNEL_ON;
		if(!useGCS)
			sprintf(temp_c,"%d",3);
		else
			sprintf(temp_c,"%s","C 1");
	}
	else
	{
		cmd = SET_CHANNEL_OFF;
		if(!useGCS)
			sprintf(temp_c,"%d",3);
		else
			sprintf(temp_c,"%s","C 0");
	}
	if(WriteReadMercury(cmd,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::write_Speed()", Tango::ERR);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_Out04
// 
// description : 	Extract real attribute values for Out04 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_Out04(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_Out04(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Out04_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::write_Out04
// 
// description : 	Write Out04 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PiMercury::write_Out04(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PiMercury::write_Out04(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Out04_write);
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;
	if(attr_Out04_write)
	{
		cmd = SET_CHANNEL_ON;
		if(!useGCS)
			sprintf(temp_c,"%d",4);
		else
			sprintf(temp_c,"%s","D 1");
	}
	else
	{
		cmd = SET_CHANNEL_OFF;
		if(!useGCS)
			sprintf(temp_c,"%d",4);
		else
			sprintf(temp_c,"%s","D 0");
	}
	if(WriteReadMercury(cmd,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::write_Speed()", Tango::ERR);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_In01
// 
// description : 	Extract real attribute values for In01 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_In01(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_In01(Tango::Attribute &attr) entering... "<< endl;
	
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;

	cmd = TELL_CHANNEL;
	if(!useGCS)
		sprintf(temp_c,"%d",1);
	else if(c_86x_type < C_863_12)
		sprintf(temp_c,"%s","A");
	else
		sprintf(temp_c,"%s","1");

	if(WriteReadMercury(cmd,temp_c,resp, true) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}
	else if(resp.length() > 2)
	{
		int val;
		if(!useGCS)
		{
			if(sscanf(resp.c_str(), "H01:%d",&val) == 1)
			{
				*attr_In01_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
		else
		{

			if(c_86x_type < C_863_12 && sscanf(resp.c_str(), "A=%d",&val) == 1)
			{
				*attr_In01_read = (val != 0);
			}
			else if(c_86x_type == C_863_12 && sscanf(resp.c_str(), "1=%d",&val) == 1)
			{
				*attr_In01_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
	}
	else
	{
		Tango::Except::throw_exception(
			(const char *) "Parsing response failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}
	
	attr.set_value(attr_In01_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_In02
// 
// description : 	Extract real attribute values for In02 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_In02(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_In02(Tango::Attribute &attr) entering... "<< endl;
	
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;

	cmd = TELL_CHANNEL;
	if(!useGCS)
		sprintf(temp_c,"%d",2);
	else if(c_86x_type < C_863_12)
		sprintf(temp_c,"%s","B");
	else
		sprintf(temp_c,"%s","2");

	if(WriteReadMercury(cmd,temp_c,resp, true) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}
	else if(resp.length() > 2)
	{
		int val;
		if(!useGCS)
		{
			if(sscanf(resp.c_str(), "H02:%d",&val) == 1)
			{
				*attr_In02_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
		else
		{
			if(c_86x_type < C_863_12 && sscanf(resp.c_str(), "B=%d",&val) == 1)
			{
				*attr_In02_read = (val != 0);
			}
			else if(c_86x_type == C_863_12 && sscanf(resp.c_str(), "2=%d",&val) == 1)
			{
				*attr_In02_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
	}
	else
	{
		Tango::Except::throw_exception(
			(const char *) "Parsing response failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}

	attr.set_value(attr_In02_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_In03
// 
// description : 	Extract real attribute values for In03 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_In03(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_In03(Tango::Attribute &attr) entering... "<< endl;
	
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;

	cmd = TELL_CHANNEL;
	if(!useGCS)
		sprintf(temp_c,"%d",3);
	else if(c_86x_type < C_863_12)
		sprintf(temp_c,"%s","C");
	else
		sprintf(temp_c,"%s","3");

	if(WriteReadMercury(cmd,temp_c,resp, true) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}
	else if(resp.length() > 2)
	{
		int val;
		if(!useGCS)
		{
			if(sscanf(resp.c_str(), "H03:%d",&val) == 1)
			{
				*attr_In03_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
		else
		{
			if(c_86x_type < C_863_12 && sscanf(resp.c_str(), "C=%d",&val) == 1)
			{
				*attr_In03_read = (val != 0);
			}
			else if(c_86x_type == C_863_12 && sscanf(resp.c_str(), "3=%d",&val) == 1)
			{
				*attr_In03_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
	}
	else
	{
		Tango::Except::throw_exception(
			(const char *) "Parsing response failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}
	
	attr.set_value(attr_In03_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PiMercury::read_In04
// 
// description : 	Extract real attribute values for In04 acquisition result.
//
//-----------------------------------------------------------------------------
void PiMercury::read_In04(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PiMercury::read_In04(Tango::Attribute &attr) entering... "<< endl;
	
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) __func__, Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) __func__, Tango::ERR);	
	}
	string resp;
	char temp_c[32];
	string cmd;

	cmd = TELL_CHANNEL;
	if(!useGCS)
		sprintf(temp_c,"%d",4);
	else if(c_86x_type < C_863_12)
		sprintf(temp_c,"%s","D");
	else
		sprintf(temp_c,"%s","4");

	if(WriteReadMercury(cmd,temp_c,resp, true) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}
	else if(resp.length() > 2)
	{
		int val;
		if(!useGCS)
		{
			if(sscanf(resp.c_str(), "H04:%d",&val) == 1)
			{
				*attr_In04_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
		else
		{
			if(c_86x_type < C_863_12 && sscanf(resp.c_str(), "D=%d",&val) == 1)
			{
				*attr_In04_read = (val != 0);
			}
			else if(c_86x_type == C_863_12 && sscanf(resp.c_str(), "4=%d",&val) == 1)
			{
				*attr_In04_read = (val != 0);
			}
			else
			{
				Tango::Except::throw_exception(
					(const char *) "Parsing response failed",
					resp,
					(const char *) __func__, Tango::ERR);
			}
		}
	}
	else
	{
		Tango::Except::throw_exception(
			(const char *) "Parsing response failed",
			resp,
			(const char *) __func__, Tango::ERR);
	}

	attr.set_value(attr_In04_read);
}



//+------------------------------------------------------------------
/**
 *	method:	PiMercury::forward
 *
 *	description:	method to execute "Forward"
 *	move forward for argin offset
 *
 * @param	argin	relative movement in user units
 *
 */
//+------------------------------------------------------------------
void PiMercury::forward(Tango::DevDouble argin)
{
	DEBUG_STREAM << "PiMercury::forward(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::forward()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::forward()", Tango::ERR);	
	}
	if(get_state()==Tango::STANDBY)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put brake OFF before moving!",
			(const char *) "PiMercury::forward()", Tango::ERR);
	}
	else if(get_state()==Tango::OFF)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put motor loop ON before moving!",
			(const char *) "PiMercury::forward()", Tango::ERR);
	}
	else if(get_state()==Tango::DISABLE)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Reference Axis before moving!",
			(const char *) "PiMercury::forward()", Tango::ERR);
	}
	else if(get_state()==Tango::MOVING)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Wait for current movement to finish",
			(const char *) "PiMercury::forward()", Tango::ERR);
	}
	if((argin + *attr_Position_read > attr_Limits_read[1]) || (argin + *attr_Position_read < attr_Limits_read[0]))
	{
		TangoSys_MemStream out_stream;
		out_stream << "Positions allowed are in the [" << attr_Limits_read[0] << "," << attr_Limits_read[1] << "] interval" << ends;
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			out_stream.str(),
			(const char *) "PiMercury::forward()", Tango::ERR);
	}
	string resp;
	char temp_c[32];
	double temp_d;
	temp_d = argin*positionRatio;
	if(c_86x_type < C_863_11)
		sprintf(temp_c,"%d",(int)temp_d);
	else
		sprintf(temp_c,"%f",temp_d);
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	set_state(Tango::MOVING);
	readloop->mutex_sleep->unlock();	
	if(WriteReadMercury(CMD_MOV_RELATIVE,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::forward()", Tango::ERR);
	}

}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::backward
 *
 *	description:	method to execute "Backward"
 *	move backward for argin offset
 *
 * @param	argin	relative movement in user units
 *
 */
//+------------------------------------------------------------------
void PiMercury::backward(Tango::DevDouble argin)
{
	DEBUG_STREAM << "PiMercury::backward(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::backward()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::backward()", Tango::ERR);	
	}
	if(get_state()==Tango::STANDBY)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put brake OFF before moving!",
			(const char *) "PiMercury::backward()", Tango::ERR);
	}
	if(get_state()==Tango::OFF)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put motor loop ON before moving!",
			(const char *) "PiMercury::backward()", Tango::ERR);
	}
	else if(get_state()==Tango::DISABLE)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Reference Axis before moving!",
			(const char *) "PiMercury::backward()", Tango::ERR);
	}
	else if(get_state()==Tango::MOVING)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Wait for current movement to finish",
			(const char *) "PiMercury::backward()", Tango::ERR);
	}
	if((*attr_Position_read - argin < attr_Limits_read[0]) || (*attr_Position_read - argin > attr_Limits_read[1]))
	{
		TangoSys_MemStream out_stream;
		out_stream << "Positions allowed are in the [" << attr_Limits_read[0] << "," << attr_Limits_read[1] << "] interval" << ends;
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			out_stream.str(),
			(const char *) "PiMercury::forward()", Tango::ERR);
	}
	string resp;
	char temp_c[32];
	double temp_d;
	temp_d = -argin*positionRatio;
	if(c_86x_type < C_863_11)
		sprintf(temp_c,"%d",(int)temp_d);
	else
		sprintf(temp_c,"%f",temp_d);
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	set_state(Tango::MOVING);
	readloop->mutex_sleep->unlock();	
	if(WriteReadMercury(CMD_MOV_RELATIVE,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::backward()", Tango::ERR);
	}
}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::stop
 *
 *	description:	method to execute "Stop"
 *	Stop the motor smootly
 *
 *
 */
//+------------------------------------------------------------------
void PiMercury::stop()
{
	DEBUG_STREAM << "PiMercury::stop(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	bool old_rt_mode = rtmode;
	bool old_scan_mode = scan_mode;
	scan_mode = false;
	rtmode = false;
	homing = 0;
	string resp;
	if(WriteReadMercury(CMD_STOP,"",resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::stop()", Tango::ERR);
	}
	if(old_rt_mode)
	{
		int sleep_factor;
		/*if(get_state() == Tango::MOVING)
			sleep_factor = 1;
		else*/
			sleep_factor = 5;
		readloop->mutex_sleep->lock();
		readloop->pausesec_dfl = (int)(((double)(sleep_factor*th_sleep))/1000);					//poll status fast when not moving
		readloop->pausenano_dfl = (int)(((sleep_factor*th_sleep)-readloop->pausesec_dfl*1000)*1000000);		
		readloop->mutex_sleep->unlock();
	}
	if(old_scan_mode)
	{
		string resp;
		char temp_c[32];
		double temp_d;
		temp_d = speed_save*positionRatio;
		sprintf(temp_c,"%d",(int)temp_d);
		if(WriteReadMercury(SET_VELOCITY,temp_c,resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::stop()", Tango::ERR);
		}
	}
	//position_tmp = 0;
	//position_valid = false;			
}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::abort
 *
 *	description:	method to execute "Abort"
 *	Stop the motor immediately
 *
 *
 */
//+------------------------------------------------------------------
void PiMercury::abort()
{
	DEBUG_STREAM << "PiMercury::abort(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	bool old_rt_mode = rtmode;
	bool old_scan_mode = scan_mode;
	scan_mode = false;
	string resp;
	if(WriteReadMercury(CMD_ABORT,"",resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::abort()", Tango::ERR);
	}
	if(old_rt_mode)
	{
		rtmode = false;
		int sleep_factor;
		/*if(get_state() == Tango::MOVING)
			sleep_factor = 1;
		else*/
			sleep_factor = 5;
		readloop->mutex_sleep->lock();
		readloop->pausesec_dfl = (int)(((double)(sleep_factor*th_sleep))/1000);					//poll status fast when not moving
		readloop->pausenano_dfl = (int)(((sleep_factor*th_sleep)-readloop->pausesec_dfl*1000)*1000000);		
		readloop->mutex_sleep->unlock();
	}
	if(old_scan_mode)
	{
		string resp;
		char temp_c[32];
		double temp_d;
		temp_d = speed_save*positionRatio;
		sprintf(temp_c,"%d",(int)temp_d);
		if(WriteReadMercury(SET_VELOCITY,temp_c,resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::stop()", Tango::ERR);
		}
	}
}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::brake
 *
 *	description:	method to execute "Brake"
 *	Activate or deactivate the active-low motor brake output line
 *
 * @param	argin	1=Brake ON, 2=Brake OFF
 *
 */
//+------------------------------------------------------------------
void PiMercury::brake(Tango::DevBoolean argin)
{
	DEBUG_STREAM << "PiMercury::brake(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::brake()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::brake()", Tango::ERR);	
	}
	string resp;
	string cmd = argin ? CMD_BRAKE_ON : CMD_BRAKE_OFF;
	if(WriteReadMercury(cmd,"",resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::brake()", Tango::ERR);
	}
}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::go_home
 *
 *	description:	method to execute "GoHome"
 *	Move to home position
 *
 *
 */
//+------------------------------------------------------------------
void PiMercury::go_home()
{
	DEBUG_STREAM << "PiMercury::go_home(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::go_home()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::go_home()", Tango::ERR);	
	}
	if(get_state()==Tango::STANDBY)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put brake OFF before moving!",
			(const char *) "PiMercury::go_home()", Tango::ERR);
	}
	else if(get_state()==Tango::OFF)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put motor loop ON before moving!",
			(const char *) "PiMercury::go_home()", Tango::ERR);
	}
	else if(get_state()==Tango::MOVING)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Wait for current movement to finish",
			(const char *) "PiMercury::go_home()", Tango::ERR);
	}
	string resp;
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	set_state(Tango::MOVING);
	readloop->mutex_sleep->unlock();	
	/*if(WriteReadMercury(CMD_GO_HOME,"",resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::go_home()", Tango::ERR);
	}*/
	struct timeval ts;
	struct timezone tz;	
	gettimeofday(&ts, &tz);
	ts_homing.tv_sec = ts.tv_sec;
	ts_homing.tv_usec = ts.tv_usec;
	ts_homing.tv_nsec = 0;
	init_homing = true;
	homing = 1;
	
	char temp_c[32];

	if(referenceSwitch == -1)
	{
		//look for reference switch
		if(c_86x_type < C_863_11)
			sprintf(temp_c,"%d",2);			//find edge with argument 2: start in positive dir if reference line is high
		else
			strcpy(temp_c, "");
		if(WriteReadMercury(CMD_FIND_EDGE,temp_c,resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::go_home()", Tango::ERR);
		}
	}
	else if(referenceSwitch == 0)
	{
		if(c_86x_type < C_863_11)
		{
			//find edge with argument 0: positive switch
			sprintf(temp_c,"%d",0);
			if(WriteReadMercury(CMD_FIND_EDGE,temp_c,resp, false) < 0)
			{
				Tango::Except::throw_exception(
					(const char *) "Command failed",
					resp,
					(const char *) "PiMercury::go_home()", Tango::ERR);
			}
		}
		else
		{
			//look for positive switch
			if(WriteReadMercury(CMD_11_FIND_EDGE_POS,"",resp, false) < 0)
			{
				Tango::Except::throw_exception(
					(const char *) "Command failed",
					resp,
					(const char *) "PiMercury::go_home()", Tango::ERR);
			}
		}
	}
	else //if(referenceSwitch == 1)
	{
		if(c_86x_type < C_863_11)
		{
			//find edge with argument 1: negative switch
			sprintf(temp_c,"%d",1);
			if(WriteReadMercury(CMD_FIND_EDGE,temp_c,resp, false) < 0)
			{
				Tango::Except::throw_exception(
					(const char *) "Command failed",
					resp,
					(const char *) "PiMercury::go_home()", Tango::ERR);
			}
		}
		else
		{
			//look for negative switch
			if(WriteReadMercury(CMD_11_FIND_EDGE_NEG,"",resp, false) < 0)
			{
				Tango::Except::throw_exception(
					(const char *) "Command failed",
					resp,
					(const char *) "PiMercury::go_home()", Tango::ERR);
			}
		}
	}
	
	attr_Position_write = 0;
	Tango::MultiAttribute *attr_list = get_device_attr();
	Tango::WAttribute &attr_ = attr_list->get_w_attr_by_name ("Position"); 
	attr_.set_write_value(attr_Position_write);	

}


//+------------------------------------------------------------------
/**
 *	method:	PiMercury::set_home
 *
 *	description:	method to execute "SetHome"
 *	Set current position as home
 *
 *
 */
//+------------------------------------------------------------------
void PiMercury::set_home()
{
	DEBUG_STREAM << "PiMercury::set_home(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::set_home()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::set_home()", Tango::ERR);	
	}
	string resp;

	if(c_86x_type < C_863_11)
	{
		if(WriteReadMercury(CMD_SET_HOME,"",resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::set_home()", Tango::ERR);
		}
	}
	else
	{
		if(WriteReadMercury(CMD_SET_HOME,"0",resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::set_home()", Tango::ERR);
		}
	}
}


//+------------------------------------------------------------------
/**
 *	method:	PiMercury::motor_on
 *
 *	description:	method to execute "MotorON"
 *	Sets the motor current back to the hold value.
 *
 *
 */
//+------------------------------------------------------------------
void PiMercury::motor_on()
{
	DEBUG_STREAM << "PiMercury::motor_on(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::motor_on()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::motor_on()", Tango::ERR);	
	}
	string resp;
	if(c_86x_type < C_863_11)
	{
		if(WriteReadMercury(CMD_ON,"",resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::motor_on()", Tango::ERR);
		}
	}
	else
	{
		if(WriteReadMercury(CMD_OFF,"1",resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::motor_off()", Tango::ERR);
		}
	}
}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::motor_off
 *
 *	description:	method to execute "MotorOFF"
 *	Shuts off the motor current.
 *
 *
 */
//+------------------------------------------------------------------
void PiMercury::motor_off()
{
	DEBUG_STREAM << "PiMercury::motor_off(): entering... !" << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::motor_off()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::motor_off()", Tango::ERR);	
	}
	string resp;
	if(c_86x_type < C_863_11)
	{
		if(WriteReadMercury(CMD_OFF,"",resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::motor_off()", Tango::ERR);
		}
	}
	else
	{
		if(WriteReadMercury(CMD_OFF,"0",resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::motor_off()", Tango::ERR);
		}
	}
}


//+------------------------------------------------------------------
/**
 *	method:	PiMercury::get_version
 *
 *	description:	method to execute "GetVersion"
 *	Return firmware version
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevString PiMercury::get_version()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	DEBUG_STREAM << "PiMercury::get_version(): entering... version=" << version << endl;
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	Tango::DevString	argout;
	argout = CORBA::string_alloc(version.length());
	strcpy(argout, version.c_str());
	return argout;
}



//+------------------------------------------------------------------
/**
 *	method:	PiMercury::find_edge
 *
 *	description:	method to execute "FindEdge"
 *	Move the motor until a transition on the reference line is detected.
 *	Argin=starting search direction:
 *	0=start in positive dir
 *	1= start in negative dir
 *	2=start in positive dir if reference line is high
 *	4=start in positive dir if reference line is low
 *
 * @param	argin	direction: 0=start in positive dir, 1= start in negative dir, 2=start in positive dir if reference line is high, 3=start in positive dir if reference line is low
 *
 */
//+------------------------------------------------------------------
void PiMercury::find_edge(Tango::DevLong argin)
{
	DEBUG_STREAM << "PiMercury::find_edge(): entering... !" << endl;

	//	Add your own code to control device here
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::find_edge()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::find_edge()", Tango::ERR);	
	}
	if(get_state()==Tango::STANDBY)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put brake OFF before moving!",
			(const char *) "PiMercury::find_edge()", Tango::ERR);
	}
	else if(get_state()==Tango::OFF)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put motor loop ON before moving!",
			(const char *) "PiMercury::find_edge()", Tango::ERR);
	}
	else if(get_state()==Tango::MOVING)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Wait for current movement to finish",
			(const char *) "PiMercury::find_edge()", Tango::ERR);
	}
	if((argin != 0) && (argin != 1) && (argin != 2) && (argin != 3))
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Starting search direction must be 0, 1, 2, or 3",
			(const char *) "PiMercury::find_edge()", Tango::ERR);
	}
	string resp;
	char temp_c[32];
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	set_state(Tango::MOVING);
	readloop->mutex_sleep->unlock();	
	
	if(c_86x_type < C_863_11)
	{
		sprintf(temp_c,"%d",(int)argin);
		if(WriteReadMercury(CMD_FIND_EDGE,temp_c,resp, false) < 0)
		{
			Tango::Except::throw_exception(
				(const char *) "Command failed",
				resp,
				(const char *) "PiMercury::find_edge()", Tango::ERR);
		}
	}
	else
	{
		if(argin == 0)
		{
			if(WriteReadMercury(CMD_11_FIND_EDGE_POS,"",resp, false) < 0)
			{
				Tango::Except::throw_exception(
					(const char *) "Command failed",
					resp,
					(const char *) "PiMercury::find_edge()", Tango::ERR);
			}
		}
		else if(argin == 1)
		{
			if(WriteReadMercury(CMD_11_FIND_EDGE_NEG,"",resp, false) < 0)
			{
				Tango::Except::throw_exception(
					(const char *) "Command failed",
					resp,
					(const char *) "PiMercury::find_edge()", Tango::ERR);
			}
		}
	
	}
}


//+------------------------------------------------------------------
/**
 *	method:	PiMercury::limit_switch_enable
 *
 *	description:	method to execute "LimitSwitchEnable"
 *	Enable or disable software limit switch operation
 *
 * @param	argin	0=OFF 1=ON
 *
 */
//+------------------------------------------------------------------
void PiMercury::limit_switch_enable(Tango::DevBoolean argin)
{
	DEBUG_STREAM << "PiMercury::limit_switch_enable(): entering... !" << endl;

	//	Add your own code to control device here
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::limit_switch_enable()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::limit_switch_enable()", Tango::ERR);	
	}
	string resp;
	string cmd;
	cmd = argin ? SET_LIMIT_ON : SET_LIMIT_OFF;

	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	readloop->mutex_sleep->unlock();	
	if(WriteReadMercury(cmd,"",resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::limit_switch_enable()", Tango::ERR);
	}
}


//+------------------------------------------------------------------
/**
 *	method:	PiMercury::limit_switch_level
 *
 *	description:	method to execute "LimitSwitchLevel"
 *	Sets the controller to expect both limit switch to be active-low or active-high
 *
 * @param	argin	0=LOW 1=HIGH
 *
 */
//+------------------------------------------------------------------
void PiMercury::limit_switch_level(Tango::DevBoolean argin)
{
	DEBUG_STREAM << "PiMercury::limit_switch_level(): entering... !" << endl;

	//	Add your own code to control device here
	strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::limit_switch_level()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::limit_switch_level()", Tango::ERR);	
	}
	string resp;
	string cmd;
	cmd = argin ? SET_LIMIT_HIGH : SET_LIMIT_LOW;

	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	readloop->mutex_sleep->unlock();	
	if(WriteReadMercury(cmd,"",resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::limit_switch_level()", Tango::ERR);
	}
}


//+------------------------------------------------------------------
/**
 *	method:	PiMercury::load_config
 *
 *	description:	method to execute "LoadConfig"
 *	load config file for C863.11 controllers
 *
 *
 */
//+------------------------------------------------------------------
void PiMercury::load_config()
{
	DEBUG_STREAM << "PiMercury::load_config(): entering... !" << endl;

	//	Add your own code to control device here
	if(axisConfFile.length() > 0)
		load_conf_file(axisConfFile);
	else
		Tango::Except::throw_exception(
			(const char *) "File not found",
			(const char *) "Config File not defined",
			(const char *) "PiMercury::load_config()", Tango::ERR);
}


//+------------------------------------------------------------------
//
//	method:			PiMercury::WriteReadMercury
//
//	description:	
//
//-------------------------------------------------------------------
int PiMercury::WriteReadMercury(string cmd, string value, string &argout, bool has_resp)
{
	DEBUG_STREAM << "PiMercury::WriteReadMercury(): entering... cmd=" << cmd << " value=" << value << endl;
	mutex->lock();
	if(rtmode)
	{
		argout = "Not allowed reading/writing from serial in RT mode!!";
		DEBUG_STREAM << "PiMercury::WriteReadMercury: cmd=" << cmd << ", Not allowed reading/writing from serial in RT mode!!" << endl;	
		mutex->unlock();
		return -1;
	}
	char outbuf[256],
	inbuf[256];
	stringstream temp;

	argout = string("");

	//-------------------------------------------------------------------
	// ###################### RECONNECTION START #######################
	//-------------------------------------------------------------------	
	if(err_connection)
	{
#if 0
		if(useGCS)
		{
			try	{
				delete devserial;
				abort_sleep(2*((double)wr_sleep)/1000);
				devserial = new Tango::DeviceProxy(serialdevice);
				DEBUG_STREAM << "GGG: recreated DeviceProxy";
			}
			catch(Tango::DevFailed e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error re-connecting "<<serialdevice<<" err=" << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;
			}
		}
#endif
		DEBUG_STREAM << "GGG: connection ERROR";
		Tango::DeviceData Din;
		Tango::DevLong flush = 2;  //flush input and output	
		Din << flush;
		try	{
			devserial->command_inout("DevSerFlush",Din);
		}
		catch(Tango::DevFailed e) {
			TangoSys_MemStream out_stream;
			out_stream << "Error flushing serial device = " << e.errors[0].desc << ends;
			WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
			argout = out_stream.str();
			err_connection = true;
			mutex->unlock();
			return -1;				
		}


		c_86x_type = 0;
		abort_sleep(2*((double)wr_sleep)/1000);
		if(useGCS)
		{
			c_86x_type = C_863_11;
			sprintf(outbuf,"%s\n", IDN_11_GET);
			DEBUG_STREAM << __func__<<"reconnection, starting with '"<<outbuf<<"'";
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(2*((double)wr_sleep)/1000);  //abort_sleep(sec)			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing idn get, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
			memset(inbuf,0,256);	
			try {	
				read_serial(inbuf);			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error cmd '" << outbuf << "' failed!!, err reading response= " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
			}
			int resp_len = strlen(inbuf);
			if(resp_len > 0)
			{
				string resp = string(inbuf);
				if(resp.find("C-863.11") != string::npos)
				{
					c_86x_type = C_863_11;
					INFO_STREAM << "PiMercury::PiMercury(): found type of controller C-863.11!!" << endl;
				}
				else if(resp.find("C-863.12") != string::npos)
				{
					c_86x_type = C_863_12;
					INFO_STREAM << "PiMercury::PiMercury(): found type of controller C-863.12!!" << endl;
				}
			}
			abort_sleep(((double)wr_sleep)/1000);
		}
		if(c_86x_type < C_863_11)
		{	
		//-------------------------------------------------------------------
		//						SELECT THIS CONTROLLER
		//-------------------------------------------------------------------
			sprintf(outbuf,"%s%d", ADDRESS_SELECTION_CODE,(int)axisNumber-1);	
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing address selection code, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
			//-------------------------------------------------------------------
			//							GET VERSION
			//-------------------------------------------------------------------
			sprintf(outbuf,"%s\r", VERSION_GET);	
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error getting version, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
			memset(inbuf,0,256);	
			try {	
				read_serial(inbuf);			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error cmd '" << outbuf << "' failed!!, err reading response= " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();	
				err_connection = true;
				mutex->unlock();
				return -1;		
			}
			int resp_len = strlen(inbuf);
			if(resp_len < 4 || (inbuf[resp_len-3] != '\r') || (inbuf[resp_len-2] != '\n') || (inbuf[resp_len-1] != '\x3'))
			{
				TangoSys_MemStream out_stream;
				out_stream << "Error cmd '" << outbuf << " incorrect terminator in response=" << inbuf << ends;	
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				mutex->unlock();
				return -1;		
			}
			else
			{
				resp_len -= 3;
				inbuf[resp_len] = 0;
				version = string(inbuf);
				if(version.find("C-863") != string::npos)
				{
					c_86x_type = C_863;
					INFO_STREAM << "PiMercury::WriteReadMercury(): found type of controller C-863!!" << endl;
				}
				/*else if(version.find("C-862") != string::npos)
				{
					c_86x_type = C_862;
					INFO_STREAM << "PiMercury::WriteReadMercury(): found type of controller C-862!!" << endl;
				}*/
				else
				{
					c_86x_type = C_862;
					INFO_STREAM << "PiMercury::WriteReadMercury(): type of controller not found in version, setting C-862!!" << endl;
				}
				strcpy(error_attr,errors[0][c_86x_type]);		//reset error message	
			}
		}
		else
		{
			sprintf(outbuf,"%s\n", VERSION_11_GET);	
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error getting version, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
			memset(inbuf,0,256);	
			try {	
				read_serial(inbuf);			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error cmd '" << outbuf << "' failed!!, err reading response= " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();	
				err_connection = true;
				mutex->unlock();
				return -1;		
			}
			int resp_len = strlen(inbuf);
			if(resp_len > 0)
			{
				resp_len -= 1;
				inbuf[resp_len] = 0;
				version = string(inbuf);
				DEBUG_STREAM << "PiMercury::WriteReadMercury(): version=" << version << endl;
				//strcpy(error_attr,errors[0][c_86x_type]);		//reset error message
			}
			abort_sleep(((double)wr_sleep)/1000);
			sprintf(outbuf,"%s\n", TELL_11_REFERENCED);	
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error getting version, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
			memset(inbuf,0,256);	
			try {	
				read_serial(inbuf);			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error cmd '" << outbuf << "' failed!!, err reading response= " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();	
				err_connection = true;
				mutex->unlock();
				return -1;		
			}
			resp_len = strlen(inbuf);
			if(resp_len > 0)
			{
				resp_len -= 1;
				inbuf[resp_len] = 0;
				int ax;
				int ref=0;
				if(sscanf(inbuf, "%d=%d",&ax, &ref) == 2)
				{
					if(ref != 1)
					{
						referenced = false;
					}
					else
					{
						referenced = true;
					}
					DEBUG_STREAM << "PiMercury::WriteReadMercury(): read referenced: " << ref << endl;
				}
			}
		}
		//-------------------------------------------------------------------
		//							BRAKE ON
		//-------------------------------------------------------------------	
		if(c_86x_type == C_862)
		{
			//abort_sleep(((double)wr_sleep)/1000);
			sprintf(outbuf,"%s\r", CMD_BRAKE_ON);	
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing brake ON, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
		}
		//-------------------------------------------------------------------
		//							LOAD CONFIG
		//-------------------------------------------------------------------	
		else if(c_86x_type == C_863)
		{
			if(axisConfFile.length() > 0)
			{
				
				string line;
				char temp_c[32];
				ifstream infile (axisConfFile.c_str(), ios_base::in);
				while (getline(infile, line, '\n'))
				{
					DEBUG_STREAM << __FUNCTION__ << " now line is " << line;
					try {
						if(c_86x_type < C_863_11)
						{
							sprintf(temp_c, "%s\r",line.c_str());
						}
						else
						{
							sprintf(temp_c, "%s\n",line.c_str());
						}
						write_serial(temp_c,strlen(temp_c));
					}
					catch(Tango::DevFailed &e) {
						TangoSys_MemStream out_stream;
						out_stream << "Error writing cmd = " << line << ", err = " << e.errors[0].desc << ends;
						WARN_STREAM << "PiMercury::load_conf_file(): " << out_stream.str() << endl;
						break;
					}
				}
				infile.close();
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)
			}
		}
		else if(c_86x_type == C_863_11 || c_86x_type == C_863_12)
		{
			sprintf(outbuf,"%s %ld 1\r", CMD_11_SVO, axisNumber);	
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing SVO ON, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
		}
		//-------------------------------------------------------------------
		//							SET DEFAULT SPEED
		//-------------------------------------------------------------------		
		if(defaultMaxSpeed > 0 && defaultMaxSpeed <= maxAllowedSpeed)
		{
			abort_sleep(((double)wr_sleep)/1000);
			double temp_d;
			temp_d = defaultMaxSpeed*positionRatio;
			if(c_86x_type < C_863_11)
				sprintf(outbuf,"%s%d\r", SET_VELOCITY,(int)temp_d);
			else
				sprintf(outbuf,"%s %ld %d\r", cmd_map[SET_VELOCITY].c_str(), axisNumber, (int)temp_d);
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)
				attr_Speed_write = defaultMaxSpeed;
				Tango::MultiAttribute *attr_list = get_device_attr();
				Tango::WAttribute &attr_ = attr_list->get_w_attr_by_name ("Speed"); 
				attr_.set_write_value(attr_Speed_write);		
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error setting default speed, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
		}
		//-------------------------------------------------------------------
		//							SET DEFAULT ACCELERATION
		//-------------------------------------------------------------------
		if(defaultAcceleration > 0)
		{
			double temp_d;
			temp_d = defaultAcceleration*positionRatio;
			if(c_86x_type < C_863_11)
				sprintf(outbuf,"%s%d\r", SET_ACCELERATION,(int)temp_d);
			else
				sprintf(outbuf,"%s %ld %d\r", cmd_map[SET_ACCELERATION].c_str(), axisNumber, (int)temp_d);
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)
				attr_Acceleration_write = defaultAcceleration;
				Tango::MultiAttribute *attr_list = get_device_attr();
				Tango::WAttribute &attr_ = attr_list->get_w_attr_by_name ("Acceleration"); 
				attr_.set_write_value(attr_Acceleration_write);		
			}		
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error setting default acceleration, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
		}
		//------------- TEST: try now to set brake off ----------
		//-------------------------------------------------------------------
		//							BRAKE OFF
		//-------------------------------------------------------------------	
		if(c_86x_type == C_862)
		{
			//abort_sleep(((double)wr_sleep)/1000);
			sprintf(outbuf,"%s\r", CMD_BRAKE_OFF);	
			try {	
				write_serial(outbuf,strlen(outbuf));	
				abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)			
			}
			catch(Tango::DevFailed &e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error writing brake OFF, err = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
				argout = out_stream.str();
				err_connection = true;
				mutex->unlock();
				return -1;							
			}
		}
		err_connection = false;		
	}
	//-------------------------------------------------------------------
	// ####################### RECONNECTION END ########################
	//-------------------------------------------------------------------
	//DEBUG_STREAM << __func__<<"now  c_86x_type="<<(int)c_86x_type;
	if(c_86x_type < C_863_11)
	{
		if(cmd == ADDRESS_SELECTION_CODE)
			sprintf(outbuf,"%s%s", cmd.c_str(), value.c_str());
		else if(cmd == IDN_11_GET)
			sprintf(outbuf,"%s%s\n", cmd.c_str(), value.c_str());
		else
			sprintf(outbuf,"%s%s\r", cmd.c_str(), value.c_str());
		//DEBUG_STREAM << __func__<<"now  c_86x_type="<<(int)c_86x_type << " prepared outbuf '"<<outbuf<<"'";
	}
	else
	{
		if(cmd_map.find(cmd) == cmd_map.end())
		{
			/*TangoSys_MemStream out_stream;
			out_stream << "Error writing cmd = " << cmd << ", err = cmd not supported in C_863.11" << ends;
			WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
			argout = out_stream.str();
			err_connection = true;
			mutex->unlock();
			return -1;*/
			sprintf(outbuf,"%s%s\n", cmd.c_str(), value.c_str());
			DEBUG_STREAM << __func__<<"C_863.11 but cmd '"<<cmd.c_str()<<"' not found, sending '"<<outbuf<<"'";
		}
		else if(cmd == TELL_STATUS)
		{
			//sprintf(outbuf,"%s\n", cmd_map[cmd].c_str());
			sprintf(outbuf,"SRG? 1 1\n");
			DEBUG_STREAM << __func__<<"C_863.11 cmd '"<<cmd_map[cmd].c_str()<<"', sending '"<<outbuf<<"'";
		}
		else if(cmd == TELL_CHANNEL && c_86x_type == C_863_12)
		{
			//sprintf(outbuf,"%s\n", cmd_map[cmd].c_str());
			sprintf(outbuf,"%s %s\n", cmd_map[cmd].c_str(), value.c_str());
			DEBUG_STREAM << __func__<<"C_863.12 cmd '"<<cmd_map[cmd].c_str()<<"', sending '"<<outbuf<<"'";
		}
		else
		{
			if(!value.empty())
				sprintf(outbuf,"%s %ld %s\n", cmd_map[cmd].c_str(), axisNumber, value.c_str());
			else
				sprintf(outbuf,"%s %ld\n", cmd_map[cmd].c_str(), axisNumber);
			DEBUG_STREAM << __func__<<"C_863.11 cmd '"<<cmd_map[cmd].c_str()<<"', sending '"<<outbuf<<"'";
		}
	}
	int cmd_len = strlen(outbuf);
	try {	
		write_serial(outbuf,cmd_len);	
		abort_sleep(((double)wr_sleep)/1000);  //abort_sleep(sec)	
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Error writing cmd = " << cmd << ", err = " << e.errors[0].desc << ends;
		WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
		argout = out_stream.str();
		err_connection = true;
		mutex->unlock();
		return -1;							
	}
	if(!has_resp)
	{
		if(cmd == CMD_ABORT)
			abort_sleep(0.5);  //abort_sleep(sec)		//try to sleep a little bit more if abort
		/*if(cmd.find(CMD_FIND_EDGE) != string::npos)		//needed if homing
		{
			abort_sleep(1.0);  //abort_sleep(sec)		//try to sleep a little bit more if homing
			WARN_STREAM << "PiMercury::WriteReadMercury(): homing, sleeping a little bit..." << endl;
		}*/
		mutex->unlock();
		return 0;
	}
	memset(inbuf,0,256);	
	try {	
		read_serial(inbuf);			
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Error cmd '" << outbuf << "' failed!!, err reading response= " << e.errors[0].desc << ends;
		argout = out_stream.str();
		//if checking for trigger this is not an error
		if((cmd.find(CMD_WAIT_ON) == string::npos) && (cmd.find(CMD_WAIT_OFF) == string::npos))
		{
			WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
			err_connection = true;
		}
		else
		{
			checking_trigger = false;
			DEBUG_STREAM << "PiMercury::WriteReadMercury(): checking for trigger -> " << out_stream.str() << endl;
			abort_sleep(0.5);  //abort_sleep(sec)		//try to sleep a little bit more if just checked for trigger
		}
		if(exiting_rt_mode)
		{
			exiting_rt_mode = false;
			err_connection = false;
			abort_sleep(0.6);  //abort_sleep(sec)		//try to sleep a little bit more if just exited from rt_mode
		}
		mutex->unlock();
		return -1;		
	}
	int resp_len = strlen(inbuf);
	INFO_STREAM << __func__ << ": resp_len="<< resp_len << " cmd_len=" << cmd_len;
	if(removeEcho && resp_len > cmd_len)
	{
		memcpy(inbuf, &inbuf[cmd_len], resp_len - cmd_len);
		INFO_STREAM << __func__ << ": removed echo from response len resp_len="<< resp_len << " -> " << resp_len - cmd_len;
		resp_len = resp_len - cmd_len;
	}
	if((resp_len < 4 || (inbuf[resp_len-3] != '\r') || (inbuf[resp_len-2] != '\n') || (inbuf[resp_len-1] != '\x3')) && !(cmd == IDN_11_GET || (c_86x_type == C_863_11) || (c_86x_type == C_863_12)))
	{
		DEBUG_STREAM << __func__<<": now  c_86x_type="<<(int)c_86x_type << " cmd='"<<cmd.c_str()<<"' cmd != IDN_11_GET="<<int(cmd != IDN_11_GET);
		TangoSys_MemStream out_stream;
		out_stream << "11111 Error cmd '" << cmd << value << " incorrect terminator in response=" << inbuf << ends;	
		WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;
		argout = out_stream.str();
		mutex->unlock();
		return -1;		
	}
	else if((cmd != IDN_11_GET) && (c_86x_type < C_863_11))
	{
		resp_len -= 3;
		inbuf[resp_len] = 0;
		//DEBUG_STREAM << "PiMercury::WriteReadMercury(): send cmd: " << cmd << " read response: " << inbuf << endl;
	}
	else
	{
		resp_len -= 1;
		inbuf[resp_len] = 0;
		//DEBUG_STREAM << "PiMercury::WriteReadMercury(): send cmd: " << cmd << " read response: " << inbuf << endl;
	}
	argout = string(inbuf);
	DEBUG_STREAM << "PiMercury::WriteReadMercury(): send cmd: " << cmd << " argout: " << argout << endl;
	if(cmd == TELL_STATUS && (c_86x_type < C_863_11))
	{
		if(argout.find("S:") == string::npos)
		{
			DEBUG_STREAM << "PiMercury::WriteReadMercury(): wrong response ->flushing" << endl;
			Tango::DeviceData Din;
			Tango::DevLong flush = 2;  //flush RX and TX	
			Din << flush;
			try	{
				devserial->command_inout("DevSerFlush",Din);
			}
			catch(Tango::DevFailed e) {
				TangoSys_MemStream out_stream;
				out_stream << "Error flushing serial device = " << e.errors[0].desc << ends;
				WARN_STREAM << "PiMercury::WriteReadMercury(): " << out_stream.str() << endl;		
			}
		}
	}
	mutex->unlock();
	return 0;
}

//+------------------------------------------------------------------
//
//	method:			PiMercury::RTReadMercury
//
//	description:	
//
//-------------------------------------------------------------------
int PiMercury::RTReadMercury()
{
#ifdef _SERVER_RT
	mutex->lock();
	char inbuf[2048];
	string resp;
	memset(inbuf,0,2048);
	try {	
		read_serial(inbuf);			
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Real time Position read failed!!, err= " << e.errors[0].desc << ends;	
		WARN_STREAM << "PiMercury::RTReadMercury(): " << out_stream.str() << endl;
	}
	//DEBUG_STREAM << "PiMercury::RTReadMercury(): read resp=" << inbuf << " len=" << strlen(inbuf) << endl;
	//resp = temp_rt_resp + string(inbuf);
	//int resp_len = resp.length();
	strcat(temp_rt_resp,inbuf);
	int resp_len = strlen(temp_rt_resp);
	if(resp_len < 4)// || (inbuf[resp_len-3] != '\r') || (inbuf[resp_len-2] != '\n') || (inbuf[resp_len-1] != '\x3'))
	{
		TangoSys_MemStream out_stream;
		out_stream << "RT read resp len=" << resp_len << " resp=" << inbuf << ends;	
		DEBUG_STREAM << "PiMercury::RTReadMercury(): " << out_stream.str() << endl;
		//temp_rt_resp = string(inbuf);		//reading with DevSerReadChar and SL_LINE it shouldn't be useful
	}
	else
	{
		int temp_len = resp_len;
		char *ptr_end = &temp_rt_resp[0];
		char *ptr_start = &temp_rt_resp[0];
		int pos;
		char pos_str[128];
		//DEBUG_STREAM << "PiMercury::RTReadMercury(): read resp+temp=" << temp_rt_resp << " len=" << resp_len << endl;
		while(temp_len >= 3)
		{
			if((*ptr_end == '\r') && (*(ptr_end+1) == '\n') && (*(ptr_end+2) == '\x3'))
			{
				//DEBUG_STREAM << "PiMercury::RTReadMercury(): found pos temp_len=" << temp_len << endl;
				memset(pos_str,0,sizeof(pos_str));
				strncpy(pos_str,ptr_start,(ptr_end-ptr_start));
				sscanf(pos_str, "P:%d",&pos);
				if(rt_pos_array_index < rtArraySize)
				{
					rt_pos_array.pos[rt_pos_array_index] = (double)(pos) / positionRatio;
					*attr_Position_read = rt_pos_array.pos[rt_pos_array_index];
					//DEBUG_STREAM << "PiMercury::RTReadMercury(): read pos=" << rt_pos_array.pos[rt_pos_array_index] << endl;
					struct timeval tv;
					struct timezone tz;	
					gettimeofday(&tv, &tz);
					rt_pos_array.ts[rt_pos_array_index].tv_sec = tv.tv_sec;
					rt_pos_array.ts[rt_pos_array_index].tv_usec = tv.tv_usec;
					rt_pos_array.ts[rt_pos_array_index].tv_nsec = 0;
					if(rt_pos_array_index == 0)
						rt_pos_array.bunch[rt_pos_array_index] = 1;												//TODO: TEMP!!!
					else
						rt_pos_array.bunch[rt_pos_array_index] = rt_pos_array.bunch[rt_pos_array_index-1] + 1;	//TODO: TEMP!!!
					rt_pos_array_index++;
				}
				ptr_end+=3;
				ptr_start = ptr_end;
				temp_len-=3;
			}
			else
			{
				ptr_end++;
				temp_len--;
				//DEBUG_STREAM << "PiMercury::RTReadMercury(): looking for pos temp_len=" << temp_len << " ptr->" << hex << *ptr_end << "," << *(ptr_end+1) << "," << *(ptr_end+2) << endl;
			}
		}
		if(temp_len > 0)
		{
			char temp[128];
			//DEBUG_STREAM << "PiMercury::RTReadMercury(): remained for pos temp_len=" << temp_len << endl;
			strcpy(temp,&temp_rt_resp[resp_len-temp_len]);
			strcpy(temp_rt_resp,temp);
		}
		else
			strcpy(temp_rt_resp,"");
	}
	mutex->unlock();
#else
	abort_sleep((2*(double)th_sleep)/1000);
#endif
	return 0;
}

//+------------------------------------------------------------------
//
//	method:			PiMercury::write_serial
//
//	description:	method to execute "write_serial"
//					Write a string to serial interface
//
//-------------------------------------------------------------------
void PiMercury::write_serial(const char *write_str, long size)
{
	//DEBUG_STREAM << "PiMercury::write_serial(): entering... !" << endl;

	//	Add your own code to control device here
	Tango::DeviceData Din,Din2;

//	Tango::DevLong flush = 2;
	Tango::DevVarCharArray *input = new Tango::DevVarCharArray();
	input->length(size);
//	input->length(write_str.length());
	stringstream temp;
	
//	for (int i=0; i < write_str.length(); i++)
	for (int i=0; i < size; i++)
	{
		//(*input)[i] = (unsigned char) write_str.at(i);
		(*input)[i] = write_str[i];
		temp << showbase << hex << (int)(*input)[i] << " ";
	}
	DEBUG_STREAM << "PiMercury::write_serial(): writing=" << temp.str() << endl;

	try {
		Din << input;
		devserial->command_inout("DevSerWriteChar",Din);
/*		Din2 << flush;
		devserial->command_inout("DevSerFlush",Din2);*/
	}
	catch(Tango::DevFailed e) {	
			TangoSys_MemStream out_stream;
			out_stream << "Write command to SerialDeviceServer failed" << ends;
			INFO_STREAM << "PiMercury::write_serial(): " << out_stream.str() << write_str << " error=" << e.errors[0].desc << endl;
			set_state(Tango::FAULT);
			set_status("Errors from serialdevice " + serialdevice);		
			Tango::Except::re_throw_exception(e,
				(const char *) "Error writing serial",
				out_stream.str(),
				(const char *) "PiMercury::write_serial()", Tango::ERR);
	}
	
//	set_state(Tango::ON);
//	set_status("Connected to serialdevice " + serialdevice);
}

//+------------------------------------------------------------------
//
//	method:			PiMercury::read_serial
//
//	description:	method to execute "read_serial"
//					Read from serial interface 
//
//-------------------------------------------------------------------
void PiMercury::read_serial(char *read_str/*, long read_len*/)
{
	//DEBUG_STREAM << "PiMercury::read_serial(): entering... !" << endl;

	//	Add your own code to control device here
	Tango::DevVarLongStringArray strin;
	string input, strout, strout_filt;
	Tango::DeviceData Din,Dout;
	const Tango::DevVarCharArray *out;
	//long attr_read_param = SL_RAW;
	long attr_read_param;
	//attr_read_param = SAALEN << 8;
	//attr_read_param = 1 << 8;   //provo a leggere solo un carattere
	//attr_read_param |= SL_NCHAR;
	attr_read_param = SL_LINE;
	
	//attr_read_param = 2 << 8;   //provo a leggere per 2 tentativi
	//attr_read_param |= SL_RETRY;
	long temp_len;

	try {	
		Din << attr_read_param;
		Dout = devserial->command_inout("DevSerReadChar",Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "DevSerReadNChar command to SerialDeviceServer failed" << ends;
		//do not set_state here if check_trigger cmd
		if(!checking_trigger && !exiting_rt_mode)
		{
			set_state(Tango::FAULT);
			set_status("Errors from serialdevice " + serialdevice);
		}
		WARN_STREAM << "PiMercury::read_serial(): " <<  out_stream.str() << endl;			
		Tango::Except::re_throw_exception(e,
			(const char *) "Error reading serial",
			out_stream.str(),
			(const char *) "PiMercury::read_serial()", Tango::ERR);
	}
	if(!Dout.is_empty())
		Dout >> out;
	else
	{
		//set_state(Tango::FAULT);
		//set_status("Errors from serialdevice " + serialdevice);			
		Tango::Except::throw_exception(
			(const char *) "Serialline timeout",
			(const char *) "No data from serialline",
			(const char *) "PiMercury::read_serial()", Tango::ERR);		
	}

	memset(read_str,0,256);
	temp_len = out->length();
	if(temp_len > 256)	
	{
		WARN_STREAM << "PiMercury::read_serial(): read buffer overflow, " << 
			temp_len << " > " <<  256 << endl;
		temp_len = 256;
	}
	stringstream temp;
	for (int i = 0; i < temp_len; i++)
	{
		read_str[i] = (*out)[i];
		temp << showbase << hex << (int)(*out)[i] << " ";
	}
	DEBUG_STREAM << "PiMercury::read_serial(): read=" << temp.str() << endl;
}


//+------------------------------------------------------------------
//
//	method:			PiMercury::check_timeout
//
//	description:	method to execute "check_timeout"
//					Return -1 if timeout [msec.] exceeded
//
//-------------------------------------------------------------------
int PiMercury::check_timeout(timeval time, int timeout)
{
	struct timeval timenow;
	gettimeofday(&timenow,NULL);
	
	if ((timenow.tv_sec -	time.tv_sec) > (timeout/1000)) {
		DEBUG_STREAM << "PiMercury::check_timeout(): TIMEOUT! sec diff=" << (timenow.tv_sec - time.tv_sec) << " > timeout=" << (timeout/1000) << endl;
		return -1;
	}	
	if (((timenow.tv_sec -	time.tv_sec)*1000 - 
			(int)((float)(timenow.tv_usec -	time.tv_usec)/1000)) > timeout) {		
		return -1;
	}
	return 0;
	
}	/* end check_timeout() */	


//+------------------------------------------------------------------
//
//	method:			PiMercury::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void PiMercury::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.1); i++) {
		if (abortflag)
			break;
		if(abort_rt_sleep)
			break;
		omni_thread::sleep(0,100000000);
	}

}



//+------------------------------------------------------------------
/**
 *	method:	PiMercury::get_position
 *
 *	description:	method to execute "GetPosition"
 *	Mode 0 (return last n values):
 *	long[0] = 0
 *	long[1] = number of values to be returned
 *	
 *	Mode 1 (return interval between timestamps):
 *	long[0] = 1
 *	long[1] = seconds of start timestamp
 *	long[2] = useconds of start timestamp
 *	long[3] = seconds of stop timestamp
 *	long[4] = useconds of stop timestamp
 *	
 *	Mode 2 (return interval between bunch numbers):
 *	long[0] = 2
 *	long[1] = start bunch
 *	long[2] = stop bunch
 *
 * @param	argin	mode and interval
 * @return	array of positions
 *
 */
//+------------------------------------------------------------------
Tango::DevVarDoubleArray *PiMercury::get_position(const Tango::DevVarLongArray *argin)
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	//argout->length(1);
	//(*argout)[0] = 0.0;
	TangoSys_MemStream out_stream;
	DEBUG_STREAM << "PiMercury::get_position(): entering... argin->length=" << argin->length()  << " mode=" << (*argin)[0] << endl;
	if (argin->length() < 2)
		Tango::Except::throw_exception(
			(const char *) "Argument error",
			(const char *) "Command accepts at least 2 arguments",
			(const char *) "PiMercury::get_position()", Tango::ERR);
#if 0	//old rt_buffer modes
	if(((*argin)[0] != BUF_GET_LAST_MODE) && ((*argin)[0] != BUF_GET_TIME_MODE) && ((*argin)[0] != BUF_GET_BUNCH_MODE) && 
		((*argin)[0] != BUF_GET_LAST_UNIQUE_MODE) && ((*argin)[0] != BUF_GET_TIME_UNIQUE_MODE) && ((*argin)[0] != BUF_GET_BUNCH_UNIQUE_MODE))
		Tango::Except::throw_exception(
			(const char *) "Argument error",
			"Allowed value for argin[0]:\n - 0 = get last values\n - 1 = get values between timestamps\n - 2 = get values between bunch numbers\n current value=" + (*argin)[0],
			(const char *) "PiMercury::get_position()", Tango::ERR);
#endif
#if 0	//old rt_buffer modes
#ifdef _SERVER_RT
	if((*argin)[0] == GET_RT_LAST_VALUES)
	{
		DEBUG_STREAM << "PiMercury::get_position(): getting last " << (*argin)[1] << " values" << endl;
		if((*argin)[1] > rt_pos_array_index)
		{
			argout->length(rt_pos_array_index+1);
			(*argout)[0] = (Tango::DevDouble)rt_pos_array_index;
			INFO_STREAM << "PiMercury::get_position(): requested " << (*argin)[1] << " values but returning " << rt_pos_array_index << endl;
			for(int i=0 ; i < rt_pos_array_index; i++)
				(*argout)[i+1] = rt_pos_array.pos[i];
		}
		else
		{
			argout->length((*argin)[1]+1);
			(*argout)[0] = (Tango::DevDouble)(*argin)[1];
			for(int i=0 ; i < (*argin)[1]; i++)
				(*argout)[i+1] = rt_pos_array.pos[i];
		}

	}
	else if((*argin)[0] == GET_RT_VALUES_TIME)
	{
		if(argin->length() == 5)
		{
			DEBUG_STREAM << "PiMercury::get_position(): getting values between " << (*argin)[1] << " s + " << (*argin)[2] << " us and " << (*argin)[3] << " s + " << (*argin)[4] << " us" << endl;
			int count = 0;
			Tango::TimeVal t_start,t_end;
			t_start.tv_sec = (*argin)[1];
			t_start.tv_usec = (*argin)[2];
			t_end.tv_sec = (*argin)[3];
			t_end.tv_usec = (*argin)[4];
			for(int i=0 ; i < rt_pos_array_index; i++)
			{
				if((tv_compare(rt_pos_array.ts[i],t_start) >= 0) && (tv_compare(rt_pos_array.ts[i],t_end) <= 0))
					count++;
			}			
			argout->length(count+1);
			(*argout)[0] = (Tango::DevDouble)count;
			INFO_STREAM << "PiMercury::get_position(): returning " << count << " values" << endl;
			int j=1;
			for(int i=0 ; i < rt_pos_array_index; i++)
			{
				if((tv_compare(rt_pos_array.ts[i],t_start) >= 0) && (tv_compare(rt_pos_array.ts[i],t_end) <= 0))
					(*argout)[j++] = rt_pos_array.pos[i];
			}
		}
		else
			Tango::Except::throw_exception(
				(const char *) "Argument error",
				"In mode 1 other 4 argins are needed",
				(const char *) "PiMercury::get_position()", Tango::ERR);
	}
	else if((*argin)[0] == GET_RT_VALUES_BUNCH)
	{
		if(argin->length() == 3)
		{
			int count = 0;
			int b_start,b_end;
			b_start = (*argin)[1];
			b_end = (*argin)[2];
			DEBUG_STREAM << "PiMercury::get_position(): getting values between bunch " << b_start << " and bunch " << b_end << endl;
			for(int i=0 ; i < rt_pos_array_index; i++)
			{
				if((rt_pos_array.bunch[i] >= b_start) && (rt_pos_array.bunch[i] <= b_end))
					count++;
			}			
			argout->length(count+1);
			(*argout)[0] = (Tango::DevDouble)count;
			INFO_STREAM << "PiMercury::get_position(): returning " << count << " values" << endl;
			int j=1;
			for(int i=0; i < rt_pos_array_index; i++)
			{
				if((rt_pos_array.bunch[i] >= b_start) && (rt_pos_array.bunch[i] <= b_end))
					(*argout)[j++] = rt_pos_array.pos[i];
			}
		}
		else
			Tango::Except::throw_exception(
				(const char *) "Argument error",
				"In mode 2 other 2 argins are needed",
				(const char *) "PiMercury::get_position()", Tango::ERR);
	}
	else if((*argin)[0] == GET_RT_LAST_TIMES)
	{
		DEBUG_STREAM << "PiMercury::get_position(): getting last " << (*argin)[1] << " timestamps" << endl;
		if((*argin)[1] > rt_pos_array_index)
		{
			argout->length(rt_pos_array_index+1);
			(*argout)[0] = (Tango::DevDouble)rt_pos_array_index;
			INFO_STREAM << "PiMercury::get_position(): requested " << (*argin)[1] << " values but returning " << rt_pos_array_index << endl;
			for(int i=0 ; i < rt_pos_array_index; i++)
			{
				(*argout)[i+1] = (double)rt_pos_array.ts[i].tv_sec + (double)rt_pos_array.ts[i].tv_usec*0.000001;
			}
		}
		else
		{
			argout->length((*argin)[1]+1);
			(*argout)[0] = (Tango::DevDouble)(*argin)[1];
			for(int i=0 ; i < (*argin)[1]; i++)
			{
				(*argout)[i+1] = (double)rt_pos_array.ts[i].tv_sec + (double)rt_pos_array.ts[i].tv_usec*0.000001;
			}
		}

	}
#else
	if(fd_buffer < 0)
	{
		TangoSys_MemStream out_stream;
		out_stream << "Error " << rTBufferdevice << " not opened!" << ends;
		Tango::Except::throw_exception(
			(const char *) "Error",
			out_stream.str(),
			(const char *) "PiMercury::get_position()", Tango::ERR);
	}
	long idx[2];
	unsigned long tmp_size;
	long *tmp_array;
	unsigned char unique;
	if((*argin)[0] == BUF_GET_LAST_MODE)
	{
		DEBUG_STREAM << "PiMercury::get_position(): BUF_GET_LAST_MODE getting last " << (*argin)[1] << " values" << endl;
		unique = 0;	//retrieve all data also with duplicated bunch numbers
		tmp_size = get_last_idx((*argin)[1], unique, idx, buf);
		DEBUG_STREAM << "PiMercury::get_position(): get_last_idx returned " << tmp_size << endl;
		if(tmp_size < (*argin)[1])
		{
			TangoSys_MemStream out_stream;
			out_stream << "Error: requested " << (*argin)[1] << " values but retrieved " << tmp_size << ends;
			DEBUG_STREAM << "PiMercury::get_position(): " << out_stream.str() << endl;
			Tango::Except::throw_exception(
				(const char *) "Buffer error",
				out_stream.str(),
				(const char *) "PiMercury::get_position()", Tango::ERR);			
		}
		argout->length(tmp_size);
		/*tmp_array = new long[tmp_size];
		//read_data(idx,&(*argout)[0],buf);
		read_data(unique, idx,&tmp_array[0],buf);
		for(int k=0; k<tmp_size; k++)
			(*argout)[k] = (tmp_array[k] != FILL_ONE_LONG) ? ((double)tmp_array[k]) / positionRatio : NAN;
			
		delete [] tmp_array;*/
		read_data_calib(unique, idx,&(*argout)[0],buf, 0, 1.0 / positionRatio, 0);
	}
	else if((*argin)[0] == BUF_GET_TIME_MODE)
	{
		unique = 0;	//retrieve all data also with duplicated bunch numbers
		if(argin->length() == 5)
		{
			DEBUG_STREAM << "PiMercury::get_position(): BUF_GET_TIME_MODE getting values between " << (*argin)[1] << " s + " << (*argin)[2] << " us and " << (*argin)[3] << " s + " << (*argin)[4] << " us" << endl;
			struct timeval t_start,t_end;
			t_start.tv_sec = (*argin)[1];
			t_start.tv_usec = (*argin)[2];
			t_end.tv_sec = (*argin)[3];
			t_end.tv_usec = (*argin)[4];
			tmp_size = get_time_idx(t_start, t_end, unique, idx, buf);
			argout->length(tmp_size);
			/*tmp_array = new long[tmp_size];
			//read_data(idx,&(*argout)[0],buf);
			read_data(unique, idx,&tmp_array[0],buf);
			for(int k=0; k<tmp_size; k++)
				(*argout)[k] = (tmp_array[k] != FILL_ONE_LONG) ? ((double)tmp_array[k]) / positionRatio : NAN;
			delete [] tmp_array;*/
			read_data_calib(unique, idx,&(*argout)[0],buf, 0, 1.0 / positionRatio, 0);
		}
		else
			Tango::Except::throw_exception(
				(const char *) "Argument error",
				"In mode 1 other 4 argins are needed",
				(const char *) "PiMercury::get_position()", Tango::ERR);
	}
	else if((*argin)[0] == BUF_GET_BUNCH_MODE)
	{
		unique = 0;	//retrieve all data also with duplicated bunch numbers
		if(argin->length() == 3)
		{
			int count = 0;
			int b_start,b_end;
			b_start = (*argin)[1];
			b_end = (*argin)[2];
			DEBUG_STREAM << "PiMercury::get_position(): BUF_GET_BUNCH_MODE getting values between bunch " << b_start << " and bunch " << b_end << endl;
			tmp_size = get_bunch_idx(b_start, b_end, unique, idx, buf);
			if(tmp_size < (b_end - b_start +1))
			{
				long pos;
				struct timeval timestamp;
				unsigned long bunchnumber;
				read_last(&pos, &timestamp, &bunchnumber,buf);
				TangoSys_MemStream out_stream;
				out_stream << "Error: requested " << b_end - b_start +1 << " values but retrieved " << tmp_size << ends;
				INFO_STREAM << "PiMercury::get_position(): " << out_stream.str() <<" read_last returned: pos="<<pos<<" time s="<<(unsigned int)timestamp.tv_sec << " us="<<(unsigned int)timestamp.tv_usec<<" bunchnum="<<bunchnumber<< endl;
				Tango::Except::throw_exception(
					(const char *) "Buffer error",
					out_stream.str(),
					(const char *) "PiMercury::get_position()", Tango::ERR);			
			}
			argout->length(tmp_size);
			tmp_array = new long[tmp_size];
			//read_data(idx,&(*argout)[0],buf);
			read_data(unique, idx,&tmp_array[0],buf);
			for(int k=0; k<tmp_size; k++)
			{
				(*argout)[k] = (tmp_array[k] != FILL_ONE_LONG) ? ((double)tmp_array[k]) / positionRatio : NAN;
				//DEBUG_STREAM << "PiMercury::get_position(): from kernel pos= " << tmp_array[k] << " to user pos=" << (*argout)[k] << endl;
			}
			delete [] tmp_array;
			//read_data_calib(unique, idx,&(*argout)[0],buf, 0, 1.0 / positionRatio, 0);
		}
		else
			Tango::Except::throw_exception(
				(const char *) "Argument error",
				"In mode 2 other 2 argins are needed",
				(const char *) "PiMercury::get_position()", Tango::ERR);
	}
	else if((*argin)[0] == BUF_GET_LAST_UNIQUE_MODE)
	{
		DEBUG_STREAM << "PiMercury::get_position(): BUF_GET_LAST_UNIQUE_MODE getting last " << (*argin)[1] << " values" << endl;
		unique = 1;		//retrieve data at 10 Hz also if buffer is filled at 50 Hz
		tmp_size = get_last_idx((*argin)[1], unique, idx, buf);
		DEBUG_STREAM << "PiMercury::get_position(): get_last_idx returned " << tmp_size << endl;
		if(tmp_size != (*argin)[1])
		{
			TangoSys_MemStream out_stream;
			out_stream << "Error: requested " << (*argin)[1] << " values but retrieved " << tmp_size << ends;
			DEBUG_STREAM << "PiMercury::get_position(): " << out_stream.str() << endl;
			Tango::Except::throw_exception(
				(const char *) "Buffer error",
				out_stream.str(),
				(const char *) "PiMercury::get_position()", Tango::ERR);			
		}
		argout->length(tmp_size);
		tmp_array = new long[tmp_size];
		//read_data(idx,&(*argout)[0],buf);
		read_data(unique, idx,&tmp_array[0],buf);
		for(int k=0; k<tmp_size; k++)
			(*argout)[k] = (tmp_array[k] != FILL_ONE_LONG) ? ((double)tmp_array[k]) / positionRatio : NAN;
		delete [] tmp_array;
		//read_data_calib(unique, idx,&(*argout)[0],buf,0,1.0/positionRatio, 0);
	}
	else if((*argin)[0] == BUF_GET_TIME_UNIQUE_MODE)
	{
		unique = 1;		//retrieve data at 10 Hz also if buffer is filled at 50 Hz
		if(argin->length() == 5)
		{
			DEBUG_STREAM << "PiMercury::get_position(): BUF_GET_TIME_UNIQUE_MODE getting values between " << (*argin)[1] << " s + " << (*argin)[2] << " us and " << (*argin)[3] << " s + " << (*argin)[4] << " us" << endl;
			struct timeval t_start,t_end;
			t_start.tv_sec = (*argin)[1];
			t_start.tv_usec = (*argin)[2];
			t_end.tv_sec = (*argin)[3];
			t_end.tv_usec = (*argin)[4];
			tmp_size = get_time_idx(t_start, t_end, unique, idx, buf);
			argout->length(tmp_size);
			tmp_array = new long[tmp_size];
			//read_data(idx,&(*argout)[0],buf);
			read_data(unique, idx,&tmp_array[0],buf);
			for(int k=0; k<tmp_size; k++)
				(*argout)[k] = (tmp_array[k] != FILL_ONE_LONG) ? ((double)tmp_array[k]) / positionRatio : NAN;
			delete [] tmp_array;
			//read_data_calib(unique, idx,&(*argout)[0],buf,0,1.0/positionRatio, 0);
		}
		else
			Tango::Except::throw_exception(
				(const char *) "Argument error",
				"In mode 1 other 4 argins are needed",
				(const char *) "PiMercury::get_position()", Tango::ERR);
	}
	else if((*argin)[0] == BUF_GET_BUNCH_UNIQUE_MODE)
	{
		unique = 1;		//retrieve data at 10 Hz also if buffer is filled at 50 Hz
		if(argin->length() == 3)
		{
			int count = 0;
			int b_start,b_end;
			b_start = (*argin)[1];
			b_end = (*argin)[2];
			DEBUG_STREAM << "PiMercury::get_position(): BUF_GET_BUNCH_UNIQUE_MODE getting values between bunch " << b_start << " and bunch " << b_end << endl;
			tmp_size = get_bunch_idx(b_start, b_end, unique, idx, buf);
			if(tmp_size != (b_end - b_start +1))
			{
				TangoSys_MemStream out_stream;
				out_stream << "Error: requested " << b_end - b_start +1 << " values but retrieved " << tmp_size << ends;
				DEBUG_STREAM << "PiMercury::get_position(): " << out_stream.str() << endl;
				Tango::Except::throw_exception(
					(const char *) "Buffer error",
					out_stream.str(),
					(const char *) "PiMercury::get_position()", Tango::ERR);			
			}
			argout->length(tmp_size);
			tmp_array = new long[tmp_size];
			//read_data(idx,&(*argout)[0],buf);
			read_data(unique, idx,&tmp_array[0],buf);
			for(int k=0; k<tmp_size; k++)
			{
				(*argout)[k] = (tmp_array[k] != FILL_ONE_LONG) ? ((double)tmp_array[k]) / positionRatio : NAN;
				//DEBUG_STREAM << "PiMercury::get_position(): from kernel pos= " << tmp_array[k] << " to user pos=" << (*argout)[k] << endl;
			}
			delete [] tmp_array;
			//read_data_calib(unique, idx,&(*argout)[0],buf,0,1.0/positionRatio, 0);
		}
		else
			Tango::Except::throw_exception(
				(const char *) "Argument error",
				"In mode 2 other 2 argins are needed",
				(const char *) "PiMercury::get_position()", Tango::ERR);
	}
#endif
	else
	{
		argout->length(0);
	}
#else	//new rt_buffer modes
	if(fd_buffer < 0)
	{
		TangoSys_MemStream out_stream;
		out_stream << "Error " << rTBufferdevice << " not opened!" << ends;
		Tango::Except::throw_exception(
			(const char *) "Error",
			out_stream.str(),
			(const char *) "PiMercury::get_position()", Tango::ERR);
	}
	long buf_idx[2], num_samples;
	unsigned char unique;

	check_get_data_param(argin, buf, buf_idx, &num_samples, &unique);
	Tango::DevVarDoubleArray	*argout  = new Tango::DevVarDoubleArray();
	argout->length(num_samples);
	read_data_calib(unique, buf_idx, &(*argout)[0], buf, 0, 1.0/positionRatio, 0);
#endif
	//	Add your own code to control device here
	DEBUG_STREAM << "PiMercury::get_position(): exiting with argout->length()= " << argout->length() << endl;
	return argout;
}


int PiMercury::tv_compare(Tango::TimeVal ts1, Tango::TimeVal ts2)
{
	if(ts1.tv_sec < ts2.tv_sec)
		return - 1;
	else if(ts1.tv_sec == ts2.tv_sec)
	{
		if(ts1.tv_usec < ts2.tv_usec)
			return -1;
		if(ts1.tv_usec == ts2.tv_usec)
			return 0;
		else
			return 1;
	}
	else
		return 1;

}




//+------------------------------------------------------------------
/**
 *	method:	PiMercury::check_trigger
 *
 *	description:	method to execute "CheckTrigger"
 *	Chek for trigger trying to wait with timeout for high level or for low level, if timeout return error
 *
 * @return	true=ok
 *
 */
//+------------------------------------------------------------------
Tango::DevBoolean PiMercury::check_trigger(/*Tango::DevLong argin*/)
{
	Tango::DevBoolean	argout = true;
	DEBUG_STREAM << "PiMercury::check_trigger(): entering... !" << endl;

	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::check_trigger()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::check_trigger()", Tango::ERR);	
	}
	/*if((argin != 0) && (argin != 1))
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Argin=0 for checking waiting low trigger, Argin=1 for checking waiting high trigger",
			(const char *) "PiMercury::check_trigger()", Tango::ERR);
	}*/
	string resp;
	char temp_c[32];
	/*if(argin == 0)
		sprintf(temp_c,"%s%d,%s",CMD_WAIT_OFF,channelTrigger,TELL_POSITION);
	else
		sprintf(temp_c,"%s%d,%s",CMD_WAIT_ON,channelTrigger,TELL_POSITION);*/
	checking_trigger = true;
	sprintf(temp_c,"%s%d,%s%d,%s%d,%s",CMD_WAIT_OFF,(int)channelTrigger,CMD_WAIT_ON,(int)channelTrigger,CMD_WAIT_OFF,(int)channelTrigger,TELL_POSITION);
	if(WriteReadMercury(temp_c, "",resp, true) < 0)
	{
		argout = false;
	}	

	return argout;
}


//+------------------------------------------------------------------
/**
 *	method:	PiMercury::start_scan
 *
 *	description:	method to execute "StartScan"
 *	Start scanning mode: move from start position to end position and repeat if if wanted
 *
 * @param	argin	start pos, stop pos, repeat
 *
 */
//+------------------------------------------------------------------
void PiMercury::start_scan(const Tango::DevVarDoubleArray *argin)
{
	DEBUG_STREAM << "PiMercury::start_scan(): entering... !" << endl;

	//	Add your own code to control device here
	if(rtmode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in RT mode",
			(const char *) "PiMercury::start_scan()", Tango::ERR);	
	}
	if(scan_mode)
	{
		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not allowed in scan mode",
			(const char *) "PiMercury::start_scan()", Tango::ERR);	
	}
	if(get_state()==Tango::STANDBY)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put brake OFF before moving!",
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	else if(get_state()==Tango::OFF)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Put motor loop ON before moving!",
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	else if(get_state()==Tango::MOVING)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Wait for current movement to finish",
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	if((argin->length() != 3))
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Expected 3 parameter: start pos, stop pos, repeat",
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	if(((*argin)[0] < attr_Limits_read[0]) || ((*argin)[0] > attr_Limits_read[1]) ||
		((*argin)[1] < attr_Limits_read[0]) || ((*argin)[1] > attr_Limits_read[1]))
	{
		TangoSys_MemStream out_stream;
		out_stream << "Position allowed are in the [" << attr_Limits_read[0] << "," << attr_Limits_read[1] << "] interval" << ends;
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			out_stream.str(),
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	if(((*argin)[2] != 0) && ((*argin)[2] != 1))
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Argin[2] = repeat flag, allowed values are 0 and 1",
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	scan_start_pos = (*argin)[0];
	scan_end_pos = (*argin)[1];
	scan_repeat = (*argin)[2];
	speed_save = *attr_Speed_read;
	string resp;
	double temp_d;
	char temp_c[32];
	//set scanning speed
	temp_d = (*attr_ScanSpeed_read)*positionRatio;
	sprintf(temp_c,"%d",(int)temp_d);
	if(WriteReadMercury(SET_VELOCITY,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	
	temp_d = scan_start_pos*positionRatio;
	sprintf(temp_c,"%d",(int)temp_d);
	INFO_STREAM << "PiMercury::start_scan writing user pos=" << attr_Position_write << " -> motor pos=" << temp_c << endl;
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = (int)(((th_sleep)-readloop->pausesec_dfl*1000)*1000000);
	moving = true;
	readloop->mutex_sleep->unlock();	
	if(WriteReadMercury(CMD_MOV_ABSOLUTE,temp_c,resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::start_scan()", Tango::ERR);
	}
	scan_mode = true;
	scan_dir = -1;
}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::string_explode
 * Explode a string 
*/
//+------------------------------------------------------------------
void PiMercury::string_explode(string str, string separator, vector<string>* results)
{
	unsigned int found;
	
	found = str.find_first_of(separator);
	while(found != string::npos) {
		if(found > 0) {
			results->push_back(str.substr(0,found));
			//DEBUG_STREAM << __FUNCTION__ << " pushed"<<str.substr(0,found);
		}
		str = str.substr(found+1);
		found = str.find_first_of(separator);
	}
	if(str.length() > 0) {
		results->push_back(str);
		//DEBUG_STREAM << __FUNCTION__ << " pushed"<<str;
	}

}

//+------------------------------------------------------------------
/**
 *	method:	PiMercury::load_conf_file
 * 	load axis configuration from file 
*/
//+------------------------------------------------------------------
int PiMercury::load_conf_file(string filename)
{
	string line;
	char temp_c[32];
	ifstream infile (filename.c_str(), ios_base::in);
	mutex->lock();
	while (getline(infile, line, '\n'))
	{
		DEBUG_STREAM << __FUNCTION__ << " now line is " << line;
		try {
			if(c_86x_type < C_863_11)
			{
				sprintf(temp_c, "%s\r",line.c_str());
			}
			else
			{
				sprintf(temp_c, "%s\n",line.c_str());
			}
			write_serial(temp_c,strlen(temp_c));
		}
		catch(Tango::DevFailed &e) {
			TangoSys_MemStream out_stream;
			out_stream << "Error writing cmd = " << line << ", err = " << e.errors[0].desc << ends;
			WARN_STREAM << "PiMercury::load_conf_file(): " << out_stream.str() << endl;
			break;
		}
	}
	usleep(500000);
	mutex->unlock();
	infile.close();
	return 0;
}

}	//	namespace
